﻿using ComPDFKit.PDFDocument;
using Compdfkit_Tools.Data;
using Compdfkit_Tools.Helper;
using Compdfkit_Tools.PDFControl;
using Compdfkit_Tools.PDFControlUI;
using ComPDFKitViewer;
using ComPDFKitViewer.AnnotEvent;
using ComPDFKitViewer.PdfViewer;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Runtime.CompilerServices;
using System.Windows;
using System.Windows.Annotations;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;

namespace Annotations
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, INotifyPropertyChanged
    {
        private PDFViewControl passwordViewer;
        private PDFViewControl pdfViewControl;
        private CPDFAnnotationControl pdfAnnotationControl = null;

        private double[] zoomLevelList = { 1f, 8f, 12f, 25, 33f, 50, 66f, 75, 100, 125, 150, 200, 300, 400, 600, 800, 1000 };

        public bool CanSave
        {
            get
            {
                if (pdfViewControl != null && pdfViewControl.PDFView != null)
                {
                    return pdfViewControl.PDFView.UndoManager.CanSave;
                }
                return false;
            }
        }

        public bool CanUndo
        {
            get
            {
                if (pdfViewControl != null && pdfViewControl.PDFView != null)
                {
                    return pdfViewControl.PDFView.UndoManager.CanUndo;
                }
                return false;
            }
        }

        public bool CanRedo
        {
            get
            {
                if (pdfViewControl != null && pdfViewControl.PDFView != null)
                {
                    return pdfViewControl.PDFView.UndoManager.CanRedo;
                }
                return false;
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
        public ICommand CloseTabCommand;
        public MainWindow()
        {
            InitializeComponent();
            DataContext = this;
        }

        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            pdfAnnotationControl = new CPDFAnnotationControl();
            BotaSideTool.AddBOTAContent(BOTATools.Thumbnail | BOTATools.Outline | BOTATools.Bookmark | BOTATools.Search | BOTATools.Annotation);
            LoadDefaultDocument();
        }

        private void LoadDocument()
        {
            pdfViewControl.PDFView?.Load();
            pdfViewControl.PDFView?.SetShowLink(true);
            PDFGrid.Child = pdfViewControl;
            pdfViewControl.PDFView.InfoChanged -= PdfViewer_InfoChanged;
            pdfViewControl.PDFView.InfoChanged += PdfViewer_InfoChanged;
            pdfViewControl.PDFView.AnnotCommandHandler -= PDFView_AnnotCommandHandler;
            pdfViewControl.PDFView.AnnotCommandHandler += PDFView_AnnotCommandHandler;
            pdfViewControl.PDFView.AnnotEditHandler -= PDFView_AnnotEditHandler;
            pdfViewControl.PDFView.AnnotEditHandler += PDFView_AnnotEditHandler;
            pdfViewControl.PDFView.UndoManager.PropertyChanged -= UndoManager_PropertyChanged;
            pdfViewControl.PDFView.UndoManager.PropertyChanged += UndoManager_PropertyChanged;
            pdfViewControl.PDFView.AnnotActiveHandler -= PDFView_AnnotActiveHandler;
            pdfViewControl.PDFView.AnnotActiveHandler += PDFView_AnnotActiveHandler;
            pdfViewControl.PDFView.SetFormFieldHighlight(true);
            PasswordUI.Closed -= PasswordUI_Closed;
            PasswordUI.Canceled -= PasswordUI_Canceled;
            PasswordUI.Confirmed -= PasswordUI_Confirmed;
            PasswordUI.Closed += PasswordUI_Closed;
            PasswordUI.Canceled += PasswordUI_Canceled;
            PasswordUI.Confirmed += PasswordUI_Confirmed;

            pdfViewControl.PDFView.ChangeFitMode(FitMode.FitWidth);
            CPDFSaclingControl.InitWithPDFViewer(pdfViewControl.PDFView);
            CPDFSaclingControl.SetZoomTextBoxText(string.Format("{0}", (int)(pdfViewControl.PDFView.ZoomFactor * 100)));

            ViewSettingBtn.IsChecked = false;
            PropertyContainer.Child = null;
            PropertyContainer.Visibility = Visibility.Collapsed;

            InitialPDFViewControl(pdfViewControl);
            FloatPageTool.InitWithPDFViewer(pdfViewControl.PDFView);
            BotaSideTool.InitWithPDFViewer(pdfViewControl.PDFView);
            BotaSideTool.SelectBotaTool(BOTATools.Thumbnail);
        }

        private void PDFView_AnnotActiveHandler(object sender, AnnotAttribEvent e)
        {
                PropertyContainer.Child = pdfAnnotationControl;
                pdfAnnotationControl.SetAnnotEventData(e);
        }

        private void PDFView_AnnotEditHandler(object sender, List<AnnotEditEvent> e)
        {
            BotaSideTool.LoadAnnotationList();
        }

        private void UndoManager_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            OnPropertyChanged(e.PropertyName);
        }

        private void PDFView_AnnotCommandHandler(object sender, AnnotCommandArgs e)
        {
            switch (e.CommandType)
            {
                case CommandType.Context:
                    e.Handle = true;
                    if (e.CommandTarget == TargetType.Annot)
                    {
                        e.Handle = true;
                        e.PopupMenu = new ContextMenu();
                        if (e.PressOnLink && AnnotationBarControl.CurrentMode == "Link")
                        {
                            e.PopupMenu.Items.Add(new MenuItem() { Header = "Delete", Command = ApplicationCommands.Delete, CommandTarget = (UIElement)sender });
                            MenuItem propertyMenu = new MenuItem();
                            propertyMenu = new MenuItem();
                            propertyMenu.Header = "Edit";
                            WeakEventManager<MenuItem, RoutedEventArgs>.AddHandler(propertyMenu, "Click", EditLink_Click);
                            propertyMenu.CommandParameter = e;
                            e.PopupMenu.Items.Add(propertyMenu);
                        }
                        else if (e.PressOnAnnot)
                        {
                            e.PopupMenu.Items.Add(new MenuItem() { Header = "Delete", Command = ApplicationCommands.Delete, CommandTarget = (UIElement)sender });
                            e.PopupMenu.Items.Add(new MenuItem() { Header = "Copy", Command = ApplicationCommands.Copy, CommandTarget = (UIElement)sender });
                            e.PopupMenu.Items.Add(new MenuItem() { Header = "Cut", Command = ApplicationCommands.Cut, CommandTarget = (UIElement)sender });
                        }
                        else if (e.PressOnMedia || e.PressOnSound)
                        {
                            e.Handle = true;
                            e.PopupMenu.Items.Add(new MenuItem() { Header = "Play", Command = MediaCommands.Play, CommandTarget = (UIElement)sender, CommandParameter = e });
                            e.PopupMenu.Items.Add(new MenuItem() { Header = "Delete", Command = ApplicationCommands.Delete, CommandTarget = (UIElement)sender });
                        }
                        else if (e.PressOnSelectedText)
                        {
                            e.PopupMenu.Items.Add(new MenuItem() { Header = "Copy", Command = ApplicationCommands.Copy, CommandTarget = (UIElement)sender });
                            MenuItem highLightMenu = new MenuItem();
                            highLightMenu.Header = "HighLight";
                            highLightMenu.Click += (o, p) =>
                            {
                                TextHighlightAnnotArgs highLightArgs = new TextHighlightAnnotArgs();
                                MouseModes oldMode = pdfViewControl.PDFView.MouseMode;

                                if (pdfAnnotationControl != null)
                                {
                                    highLightArgs.Color = System.Windows.Media.Colors.Red;
                                    highLightArgs.Transparency = 1;
                                    pdfViewControl.PDFView.SetMouseMode(MouseModes.AnnotCreate);
                                    pdfViewControl.PDFView.SetToolParam(highLightArgs);
                                    pdfViewControl.PDFView.SetMouseMode(oldMode);
                                }

                            };

                            e.PopupMenu.Items.Add(highLightMenu);

                            MenuItem underlineMenu = new MenuItem();
                            underlineMenu.Header = "UnderLine";
                            underlineMenu.Click += (o, p) =>
                            {
                                TextUnderlineAnnotArgs underlineArgs = new TextUnderlineAnnotArgs();
                                MouseModes oldMode = pdfViewControl.PDFView.MouseMode;

                                if (pdfAnnotationControl != null)
                                {
                                    underlineArgs.Color = System.Windows.Media.Colors.Red;
                                    underlineArgs.Transparency = 1;
                                    pdfViewControl.PDFView.SetMouseMode(MouseModes.AnnotCreate);
                                    pdfViewControl.PDFView.SetToolParam(underlineArgs);
                                    pdfViewControl.PDFView.SetMouseMode(oldMode);
                                }
                            };

                            e.PopupMenu.Items.Add(underlineMenu);

                            MenuItem strikeOutMenu = new MenuItem();
                            strikeOutMenu.Header = "StrikeOut";
                            strikeOutMenu.Click += (o, p) =>
                            {
                                TextStrikeoutAnnotArgs strikeoutAnnotArgs = new TextStrikeoutAnnotArgs();
                                MouseModes oldMode = pdfViewControl.PDFView.MouseMode;

                                if (pdfAnnotationControl != null)
                                {
                                    strikeoutAnnotArgs.Color = System.Windows.Media.Colors.Red;
                                    strikeoutAnnotArgs.Transparency = 1;
                                    pdfViewControl.PDFView.SetMouseMode(MouseModes.AnnotCreate);
                                    pdfViewControl.PDFView.SetToolParam(strikeoutAnnotArgs);
                                    pdfViewControl.PDFView.SetMouseMode(oldMode);
                                }
                            };

                            e.PopupMenu.Items.Add(strikeOutMenu);

                            MenuItem SquiggleMenu = new MenuItem();
                            SquiggleMenu.Header = "Squiggle";
                            SquiggleMenu.Click += (o, p) =>
                            {
                                TextSquigglyAnnotArgs squigglyAnnotArgs = new TextSquigglyAnnotArgs();
                                MouseModes oldMode = pdfViewControl.PDFView.MouseMode;

                                if (pdfAnnotationControl != null)
                                {
                                    squigglyAnnotArgs.Color = System.Windows.Media.Colors.Red;
                                    squigglyAnnotArgs.Transparency = 1;
                                    pdfViewControl.PDFView.SetMouseMode(MouseModes.AnnotCreate);
                                    pdfViewControl.PDFView.SetToolParam(squigglyAnnotArgs);
                                    pdfViewControl.PDFView.SetMouseMode(oldMode);
                                }
                            };

                            e.PopupMenu.Items.Add(SquiggleMenu);
                        }
                        else
                        {
                            e.Handle = true;
                            e.PopupMenu = new ContextMenu();

                            e.PopupMenu.Items.Add(new MenuItem() { Header = "Paste", Command = ApplicationCommands.Paste, CommandTarget = (UIElement)sender });
                            e.PopupMenu.Items.Add(new Separator());

                            MenuItem fitWidthMenu = new MenuItem();
                            fitWidthMenu.Header = "Automatically Resize";
                            fitWidthMenu.Click += (o, p) =>
                            {
                                if (pdfViewControl != null)
                                {
                                    pdfViewControl.PDFView?.ChangeFitMode(FitMode.FitWidth);
                                }
                            };

                            e.PopupMenu.Items.Add(fitWidthMenu);

                            MenuItem fitSizeMenu = new MenuItem();
                            fitSizeMenu.Header = "Actual Size";
                            fitSizeMenu.Click += (o, p) =>
                            {
                                if (pdfViewControl != null)
                                {
                                    pdfViewControl.PDFView?.ChangeFitMode(FitMode.FitSize);
                                }
                            };

                            e.PopupMenu.Items.Add(fitSizeMenu);

                            MenuItem zoomInMenu = new MenuItem();
                            zoomInMenu.Header = "Zoom In";
                            zoomInMenu.Click += (o, p) =>
                            {
                                if (pdfViewControl != null)
                                {
                                    double newZoom = CheckZoomLevel(pdfViewControl.PDFView.ZoomFactor + 0.01, true);
                                    pdfViewControl.PDFView?.Zoom(newZoom);
                                }
                            };

                            e.PopupMenu.Items.Add(zoomInMenu);

                            MenuItem zoomOutMenu = new MenuItem();
                            zoomOutMenu.Header = "Zoom Out";
                            zoomOutMenu.Click += (o, p) =>
                            {
                                if (pdfViewControl != null)
                                {
                                    double newZoom = CheckZoomLevel(pdfViewControl.PDFView.ZoomFactor - 0.01, false);
                                    pdfViewControl.PDFView?.Zoom(newZoom);
                                }
                            };

                            e.PopupMenu.Items.Add(zoomOutMenu);
                            e.PopupMenu.Items.Add(new Separator());

                            MenuItem singleView = new MenuItem();
                            singleView.Header = "Single Page";
                            singleView.Click += (o, p) =>
                            {
                                if (pdfViewControl != null)
                                {
                                    pdfViewControl.PDFView?.ChangeViewMode(ViewMode.Single);
                                }
                            };

                            e.PopupMenu.Items.Add(singleView);

                            MenuItem singleContinuousView = new MenuItem();
                            singleContinuousView.Header = "Single Page Continuous";
                            singleContinuousView.Click += (o, p) =>
                            {
                                if (pdfViewControl != null)
                                {
                                    pdfViewControl.PDFView?.ChangeViewMode(ViewMode.SingleContinuous);
                                }
                            };

                            e.PopupMenu.Items.Add(singleContinuousView);

                            MenuItem doubleView = new MenuItem();
                            doubleView.Header = "Two Pages";
                            doubleView.Click += (o, p) =>
                            {
                                if (pdfViewControl != null)
                                {
                                    pdfViewControl.PDFView?.ChangeViewMode(ViewMode.Double);
                                }
                            };

                            e.PopupMenu.Items.Add(doubleView);

                            MenuItem doubleContinuousView = new MenuItem();
                            doubleContinuousView.Header = "Two Pages Continuous";
                            doubleContinuousView.Click += (o, p) =>
                            {
                                if (pdfViewControl != null)
                                {
                                    pdfViewControl.PDFView?.ChangeViewMode(ViewMode.DoubleContinuous);
                                }
                            };

                            e.PopupMenu.Items.Add(doubleContinuousView);
                        }
                    }

                    else if (e.CommandTarget == TargetType.ImageSelection)
                    {
                        if (pdfViewControl != null && pdfViewControl.PDFView != null && pdfViewControl.PDFView.GetSelectImageCount() > 0)
                        {
                            e.Handle = true;
                            e.PopupMenu = new ContextMenu();

                            MenuItem imageCopyMenu = new MenuItem();
                            imageCopyMenu = new MenuItem();
                            imageCopyMenu.Header = "Copy Images";
                            WeakEventManager<MenuItem, RoutedEventArgs>.AddHandler(imageCopyMenu, "Click", CopyImage_Click);
                            imageCopyMenu.CommandParameter = e;
                            e.PopupMenu.Items.Add(imageCopyMenu);

                            MenuItem imageExtraMenu = new MenuItem();
                            imageExtraMenu = new MenuItem();
                            imageExtraMenu.Header = "Extract Images";
                            WeakEventManager<MenuItem, RoutedEventArgs>.AddHandler(imageExtraMenu, "Click", ExtraImage_Click);
                            imageExtraMenu.CommandParameter = e;
                            e.PopupMenu.Items.Add(imageExtraMenu);
                        }
                    }
                    break;
                case CommandType.Copy:
                    e.DoCommand();
                    break;
                case CommandType.Cut:
                case CommandType.Paste:
                case CommandType.Delete:
                    e.DoCommand();
                    break;
                default:
                    break;
            }
        }

        private void CopyImage_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Dictionary<int, List<Bitmap>> imageDict = pdfViewControl.PDFView?.GetSelectedImages();

                if (imageDict != null && imageDict.Count > 0)
                {
                    foreach (int pageIndex in imageDict.Keys)
                    {
                        List<Bitmap> imageList = imageDict[pageIndex];
                        foreach (Bitmap image in imageList)
                        {
                            MemoryStream ms = new MemoryStream();
                            image.Save(ms, System.Drawing.Imaging.ImageFormat.Png);
                            BitmapImage imageData = new BitmapImage();
                            imageData.BeginInit();
                            imageData.StreamSource = ms;
                            imageData.CacheOption = BitmapCacheOption.OnLoad;
                            imageData.EndInit();
                            imageData.Freeze();
                            Clipboard.SetImage(imageData);
                            break;
                        }
                    }
                }
            }
            catch (Exception ex)
            {

            }
        }

        private void ExtraImage_Click(object sender, RoutedEventArgs e)
        {
            System.Windows.Forms.FolderBrowserDialog folderDialog = new System.Windows.Forms.FolderBrowserDialog();
            if (folderDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                string choosePath = folderDialog.SelectedPath;
                string openPath = choosePath;
                try
                {
                    Dictionary<int, List<Bitmap>> imageDict = pdfViewControl.PDFView?.GetSelectedImages();

                    if (imageDict != null && imageDict.Count > 0)
                    {
                        foreach (int pageIndex in imageDict.Keys)
                        {
                            List<Bitmap> imageList = imageDict[pageIndex];
                            foreach (Bitmap image in imageList)
                            {
                                string savePath = Path.Combine(choosePath, Guid.NewGuid() + ".jpg");
                                image.Save(savePath, System.Drawing.Imaging.ImageFormat.Jpeg);
                                openPath = savePath;
                            }
                        }
                    }
                    Process.Start("explorer", "/select,\"" + openPath + "\"");
                }
                catch (Exception ex)
                {

                }

            }
        }

        private void PasswordUI_Confirmed(object sender, string e)
        {
            if (passwordViewer != null && passwordViewer.PDFView != null && passwordViewer.PDFView.Document != null)
            {
                passwordViewer.PDFView.Document.UnlockWithPassword(e);
                if (passwordViewer.PDFView.Document.IsLocked == false)
                {
                    PasswordUI.SetShowError("", Visibility.Collapsed);
                    PasswordUI.ClearPassword();
                    PasswordUI.Visibility = Visibility.Collapsed;
                    PopupBorder.Visibility = Visibility.Collapsed;
                    pdfViewControl = passwordViewer;
                    LoadDocument();
                }
                else
                {
                    PasswordUI.SetShowError("Wrong Password", Visibility.Visible);
                }
            }
        }

        private void PasswordUI_Canceled(object sender, EventArgs e)
        {
            PopupBorder.Visibility = Visibility.Collapsed;
            PasswordUI.Visibility = Visibility.Collapsed;
        }

        private void PasswordUI_Closed(object sender, EventArgs e)
        {
            PopupBorder.Visibility = Visibility.Collapsed;
            PasswordUI.Visibility = Visibility.Collapsed;
        }

        private void PdfViewer_InfoChanged(object sender, KeyValuePair<string, object> e)
        {

            if (e.Key == "Zoom")
            {
                CPDFSaclingControl.SetZoomTextBoxText(string.Format("{0}", (int)((double)e.Value * 100)));
            }
        }


        private double CheckZoomLevel(double zoom, bool IsGrowth)
        {
            double standardZoom = 100;
            if (zoom <= 0.01)
            {
                return 0.01;
            }
            if (zoom >= 10)
            {
                return 10;
            }

            zoom *= 100;
            for (int i = 0; i < zoomLevelList.Length - 1; i++)
            {
                if (zoom > zoomLevelList[i] && zoom <= zoomLevelList[i + 1] && IsGrowth)
                {
                    standardZoom = zoomLevelList[i + 1];
                    break;
                }
                if (zoom >= zoomLevelList[i] && zoom < zoomLevelList[i + 1] && !IsGrowth)
                {
                    standardZoom = zoomLevelList[i];
                    break;
                }
            }
            return standardZoom / 100;
        }
        private void LoadDefaultDocument()
        {
            string defaultFilePath = "developer_guide_windows.pdf";
            pdfViewControl = new PDFViewControl();
            pdfViewControl.PDFView.InitDocument(defaultFilePath);
            LoadDocument();
        }

        private void ToolExpand_Click(object sender, RoutedEventArgs e)
        {
            ToggleButton expandBtn = sender as ToggleButton;
            if (expandBtn != null)
            {
                bool isExpand = expandBtn.IsChecked == true;
                ExpandLeftPanel(isExpand);
            }
        }

        private void ExpandSearchBtn_Click(object sender, RoutedEventArgs e)
        {
            ExpandLeftPanel(true);
            BotaSideTool.SelectBotaTool(BOTATools.Search);
        }

        private void ExpandLeftPanel(bool isExpand)
        {
            BotaSideTool.Visibility = isExpand ? Visibility.Visible : Visibility.Collapsed;
            Splitter.Visibility = isExpand ? Visibility.Visible : Visibility.Collapsed;
            if (isExpand)
            {
                BodyGrid.ColumnDefinitions[0].Width = new GridLength(320);
                BodyGrid.ColumnDefinitions[1].Width = new GridLength(15);
            }
            else
            {
                BodyGrid.ColumnDefinitions[0].Width = new GridLength(0);
                BodyGrid.ColumnDefinitions[1].Width = new GridLength(0);
            }
        }

        private void ViewSettingBtn_Click(object sender, RoutedEventArgs e)
        {
            ToggleButton toggleButton = sender as ToggleButton;
            if (toggleButton != null)
            {
                if (toggleButton.IsChecked == true)
                {
                    CPDFDisplaySettingsControl displayPanel = new CPDFDisplaySettingsControl();
                    displayPanel.InitWithPDFViewer(pdfViewControl.PDFView);
                    PropertyContainer.Child = displayPanel;
                    PropertyContainer.Visibility = Visibility.Visible;
                    if ((bool)AnnotationBarBtn.IsChecked)
                    {
                        AnnotationBarBtn.IsChecked = false;
                    }
                }
                else
                {
                    PropertyContainer.Child = null;
                    PropertyContainer.Visibility = Visibility.Collapsed;
                }
            }
        }

        private void PageInfoBtn_Click(object sender, RoutedEventArgs e)
        {
            PasswordUI.Visibility = Visibility.Collapsed;
            FileInfoUI.Visibility = Visibility.Visible;
            FileInfoControl.InitWithPDFViewer(pdfViewControl.PDFView);
            PopupBorder.Visibility = Visibility.Visible;
        }

        private void FileInfoCloseBtn_Click(object sender, RoutedEventArgs e)
        {
            PopupBorder.Visibility = Visibility.Collapsed;
        }

        private void OpenFile_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                string filePath = CommonHelper.GetFilePathOrEmpty();
                if (!string.IsNullOrEmpty(filePath) && pdfViewControl != null)
                {
                    if (pdfViewControl.PDFView != null && pdfViewControl.PDFView.Document != null)
                    {
                        string oldFilePath = pdfViewControl.PDFView.Document.FilePath;
                        if (oldFilePath.ToLower() == filePath.ToLower())
                        {
                            return;
                        }
                    }

                    passwordViewer = new PDFViewControl();
                    passwordViewer.PDFView.InitDocument(filePath);
                    if (passwordViewer.PDFView.Document == null)
                    {
                        MessageBox.Show("Open File Failed");
                        return;
                    }

                    if (passwordViewer.PDFView.Document.IsLocked)
                    {
                        PasswordUI.SetShowText(System.IO.Path.GetFileName(filePath) + " password encrypted.");
                        PasswordUI.ClearPassword();
                        PopupBorder.Visibility = Visibility.Visible;
                        PasswordUI.Visibility = Visibility.Visible;
                    }
                    else
                    {
                        pdfViewControl = passwordViewer;
                        LoadDocument();
                    }
                }
            }
            catch (Exception ex)
            {

            }

        }

        private void SaveFileBtn_Click(object sender, RoutedEventArgs e)
        {
            SaveFile();
            pdfViewControl.PDFView.UndoManager.CanSave = false;
        }
        #region Annotation

        public void InitialPDFViewControl(PDFViewControl newPDFViewer)
        {
            pdfAnnotationControl.SetPDFViewer(newPDFViewer.PDFView);
            pdfAnnotationControl.AnnotationCancel();
            AnnotationBarControl.ClearAllToolState();
            ExpandRightPropertyPanel(null, Visibility.Collapsed);
            pdfAnnotationControl.ClearAnnotationBar += PdfAnnotationControl_ClearAnnotationBar;
        }

        private void PdfAnnotationControl_ClearAnnotationBar(object sender, EventArgs e)
        {
            AnnotationBarControl.ClearAllToolState();
        }
        #endregion

        private void AnnotationBarControl_Loaded(object sender, RoutedEventArgs e)
        {
            CPDFAnnotationType[] annotationProperties = { CPDFAnnotationType.Highlight, CPDFAnnotationType.Underline, CPDFAnnotationType.Strikeout, CPDFAnnotationType.Squiggly, CPDFAnnotationType.Freehand, CPDFAnnotationType.FreeText, CPDFAnnotationType.Note, CPDFAnnotationType.Circle, CPDFAnnotationType.Square, CPDFAnnotationType.Arrow, CPDFAnnotationType.Line, CPDFAnnotationType.Image, CPDFAnnotationType.Stamp, CPDFAnnotationType.Signature, CPDFAnnotationType.Link, CPDFAnnotationType.Audio };
            AnnotationBarControl.InitAnnotationBar(annotationProperties);
            AnnotationBarControl.AnnotationPropertyChanged += AnnotationBarControl_AnnotationPropertyChanged;
            AnnotationBarControl.AnnotationCancel += AnnotationBarControl_AnnotationCancel;
        }

        private void AnnotationBarControl_Unloaded(object sender, RoutedEventArgs e)
        {
            AnnotationBarControl.AnnotationPropertyChanged -= AnnotationBarControl_AnnotationPropertyChanged;
            AnnotationBarControl.AnnotationCancel -= AnnotationBarControl_AnnotationCancel;
        }

        private void AnnotationBarControl_AnnotationCancel(object sender, EventArgs e)
        {
            pdfAnnotationControl.AnnotationCancel();
            ExpandRightPropertyPanel(null, Visibility.Collapsed);
            AnnotationBarBtn.IsChecked = false;
            ViewSettingBtn.IsChecked = false;
        }

        private void AnnotationBarControl_AnnotationPropertyChanged(object sender, CPDFAnnotationType e)
        {
            //TODO:测试没有问题的话就删除
            //if(e==CPDFAnnotationType.Image)
            //{
            //    pdfViewControl.PDFView?.SetMouseMode(MouseModes.PanTool);
            //    StampAnnotArgs stampArgs = new StampAnnotArgs();
            //    stampArgs.Opacity = 1;
            //    stampArgs.Type = StampType.IMAGE_STAMP;
            //    OpenFileDialog openFileDialog = new OpenFileDialog();
            //    openFileDialog.Filter = "Image Files(*.jpg;*.jpeg;*.png;*.bmp)|*.jpg;*.jpeg;*.png;*.bmp;";
            //    if (openFileDialog.ShowDialog() == true)
            //    {
            //        stampArgs.ImagePath = openFileDialog.FileName;
                 
            //        pdfViewControl.PDFView?.SetMouseMode(MouseModes.AnnotCreate);ClearAnnotationBar
            //        pdfViewControl.PDFView?.SetToolParam(stampArgs);
            //        ExpandRightPropertyPanel(null, Visibility.Collapsed);
            //    }
            //    (sender as ToggleButton).IsChecked = false;
            //    return;
            //}
            pdfAnnotationControl.LoadAnnotationPanel(e);
            if (e != CPDFAnnotationType.Audio && e != CPDFAnnotationType.Image)
            {
                ExpandRightPropertyPanel(pdfAnnotationControl, Visibility.Visible);
                AnnotationBarBtn.IsChecked = true;
            }
        }

        private void AnnotationBarControl_Click(object sender, RoutedEventArgs e)
        {
            ToggleButton toggleButton = sender as ToggleButton;
            if (toggleButton != null)
            {
                if (toggleButton.IsChecked == true)
                {
                    if (pdfAnnotationControl != null)
                    {
                        ExpandRightPropertyPanel(pdfAnnotationControl, Visibility.Visible);
                        if ((bool)ViewSettingBtn.IsChecked)
                        {
                            ViewSettingBtn.IsChecked = false;
                        }
                    }
                }
                else
                {
                    ExpandRightPropertyPanel(null, Visibility.Collapsed);
                }
            }
        }

        public void ExpandRightPropertyPanel(UIElement propertytPanel, Visibility visible)
        {
            PropertyContainer.Width = 260;
            PropertyContainer.Child = propertytPanel;
            PropertyContainer.Visibility = visible;
            if (visible == Visibility.Collapsed || visible == Visibility.Hidden)
            {
                AnnotationBarBtn.IsChecked = false;
            }
        }

        private void EditLink_Click(object sender, RoutedEventArgs e)
        {
            PropertyContainer.Visibility = Visibility.Visible;
        }

        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var item = (sender as ComboBox).SelectedItem as ComboBoxItem;
            if ((string)item.Content == "Viewer")
            {
                AnnotationBarControl.ClearAllToolState();
                ToolBarContainer.Visibility = Visibility.Collapsed;
                ExpandRightPropertyPanel(null, Visibility.Collapsed);
                pdfAnnotationControl.AnnotationCancel();
                AnnotationBarBtn.IsChecked = false;
                if (pdfViewControl != null && pdfViewControl.PDFView != null)
                {
                    pdfViewControl.PDFView.SetMouseMode(MouseModes.Viewer);
                }
            }
            else if ((string)item.Content == "Annotation")
            {
                ToolBarContainer.Visibility = Visibility.Visible;
                if (pdfViewControl != null && pdfViewControl.PDFView != null)
                {
                    pdfViewControl.PDFView.SetMouseMode(MouseModes.PanTool);
                }
            }
        }

        private void UndoButton_Click(object sender, RoutedEventArgs e)
        {
            if (pdfViewControl != null && pdfViewControl.PDFView != null)
            {
                pdfViewControl.PDFView.UndoManager?.Undo();
            }
        }

        private void RedoButton_Click(object sender, RoutedEventArgs e)
        {
            if (pdfViewControl != null && pdfViewControl.PDFView != null)
            {
                pdfViewControl.PDFView.UndoManager?.Redo();
            }
        }

        /// <summary>
        ///触发属性更改事件通知
        /// </summary>
        protected void OnPropertyChanged([CallerMemberName] string name = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(name));
        }

        private void SaveFile()
        {
            if (pdfViewControl != null && pdfViewControl.PDFView != null && pdfViewControl.PDFView.Document != null)
            {
                try
                {
                    CPDFDocument pdfDoc = pdfViewControl.PDFView.Document;
                    if (pdfDoc.WriteToLoadedPath())
                    {
                        return;
                    }

                    SaveFileDialog saveDialog = new SaveFileDialog();
                    saveDialog.Filter = "(*.pdf)|*.pdf";
                    saveDialog.DefaultExt = ".pdf";
                    saveDialog.OverwritePrompt = true;

                    if (saveDialog.ShowDialog() == true)
                    {
                        pdfDoc.WriteToFilePath(saveDialog.FileName);
                    }
                }
                catch (Exception ex)
                {

                }
            }
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            if (pdfViewControl.PDFView.UndoManager.CanSave)
            {
                MessageBoxResult result = MessageBox.Show("Do you want to save your changes before closing the application？", "Message", MessageBoxButton.YesNoCancel);
                if (result == MessageBoxResult.Yes)
                {
                    SaveFile();
                }
                else if (result == MessageBoxResult.No)
                {

                }
                else
                {
                    e.Cancel = true;
                }
            }
        }
    }
}
