﻿using ComPDFKit.PDFAnnotation;
using ComPDFKitViewer.AnnotEvent;
using ComPDFKitViewer.PdfViewer;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using static Compdfkit_Tools.PDFControlUI.CPDFAnnoationListUI;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.StartPanel;

namespace Compdfkit_Tools.PDFControl
{
    /// <summary>
    /// CPDFAnnotationListControl.xaml 的交互逻辑
    /// </summary>
    public partial class CPDFAnnotationListControl : UserControl
    {
        /// <summary>
        /// PDFViewer
        /// </summary>
        private CPDFViewer pdfViewer;

        public CPDFAnnotationListControl()
        {
            InitializeComponent();
            Loaded += CPDFAnnotationListControl_Loaded;
        }

        private void CPDFAnnotationListControl_Loaded(object sender, RoutedEventArgs e)
        {
            AnnotationList.DeleteItemHandler -= AnnotationList_DeleteItemHandler;
            AnnotationList.DeleteItemHandler += AnnotationList_DeleteItemHandler;
        }

        private void AnnotationList_DeleteItemHandler(object sender, Dictionary<int, List<int>> e)
        {
           if(pdfViewer!=null)
            {
                pdfViewer.ClearSelectAnnots();
                pdfViewer.RemovePageAnnot(e);
                LoadAnnotationList();
            }
        }

        /// <summary>
        /// 设置PDFViewer
        /// </summary>
        public void InitWithPDFViewer(CPDFViewer newPDFView)
        {
            pdfViewer = newPDFView;
            pdfViewer.AnnotActiveHandler -= PdfViewer_AnnotActiveHandler;
            pdfViewer.AnnotActiveHandler += PdfViewer_AnnotActiveHandler;

            
        }

        private void PdfViewer_AnnotActiveHandler(object sender, AnnotAttribEvent e)
        {
            if (e != null)
            {
                //TODO SomeThing Need Change
              int a =  e.GetAnnotHandlerEventArgs(e.GetAnnotTypes())[0].AnnotIndex;
              int b =  e.GetAnnotHandlerEventArgs(e.GetAnnotTypes())[0].PageIndex;
                AnnotationList.SelectAnnotationChanged(b,a);
            }
            else
            {
                AnnotationList.SelectAnnotationChanged(-1);
            }
        }

        /// <summary>
        /// 加载缩略图
        /// </summary>
        public void LoadAnnotationList()
        {
            if (pdfViewer == null || pdfViewer.Document == null)
            {
                return;
            }

            if (pdfViewer.Document.IsLocked)
            {
                return;
            }
            int pageCount = pdfViewer.Document.PageCount;

            List<BindAnnotationResult> bindAnnotationResults = new List<BindAnnotationResult>();

            for (int i = 0; i < pageCount; i++)
            {
                List<AnnotHandlerEventArgs> annotList = pdfViewer.GetAnnotCommentList(i, pdfViewer.Document);
                if (annotList != null)
                {
                    Dispatcher.Invoke(() =>
                    {
                        foreach (AnnotHandlerEventArgs annot in annotList)
                        {
                            bindAnnotationResults.Add(new BindAnnotationResult
                            {
                                PageIndex = i,
                                annotationData = annot
                            });
                        }
                    });
                }
            }
            AnnotationList.SetAnnotationList(bindAnnotationResults);
        }

        private void AnnotationList_Loaded(object sender, RoutedEventArgs e)
        {
            AnnotationList.AnnotationSelectionChanged += AnnotationList_AnnotationSelectionChanged;
        }

        private void AnnotationList_Unloaded(object sender, RoutedEventArgs e)
        {
            AnnotationList.AnnotationSelectionChanged -= AnnotationList_AnnotationSelectionChanged;
        }

        private void AnnotationList_AnnotationSelectionChanged(object sender, object e)
        {
            var bindAnnotationResult = e as BindAnnotationResult;
            pdfViewer.SelectAnnotation(bindAnnotationResult.PageIndex, bindAnnotationResult.AnnotIndex);
        }
    }
}
