﻿using ComPDFKit.PDFDocument;
using ComPDFKitViewer.PdfViewer;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Winform = System.Windows.Forms;
using static Compdfkit_Tools.Helper.CommonHelper;
using System.Windows.Threading;
using WpfToolkit.Controls;
using Microsoft.Win32;
using System.Windows.Controls.Primitives;
using System.IO;
using System.Windows.Media.Animation;
using Compdfkit_Tools.PDFControlUI;

namespace Compdfkit_Tools.PDFControl
{
    /// <summary>
    /// CPDFPageEditControl.xaml 的交互逻辑
    /// </summary>
    public partial class CPDFPageEditControl : UserControl
    {
        #region  辅助类 辅助方法
        internal static class Utils
        {
            //根据子元素查找父元素
            public static T FindVisualParent<T>(DependencyObject obj) where T : class
            {
                while (obj != null)
                {
                    if (obj is T)
                        return obj as T;

                    obj = VisualTreeHelper.GetParent(obj);
                }
                return null;
            }

            public static childItem FindVisualChild<childItem>(DependencyObject obj)
                 where childItem : DependencyObject
            {
                for (int i = 0; i < VisualTreeHelper.GetChildrenCount(obj); i++)
                {
                    DependencyObject child = VisualTreeHelper.GetChild(obj, i);
                    if (child != null && child is childItem)
                        return (childItem)child;
                    else
                    {
                        childItem childOfChild = FindVisualChild<childItem>(child);
                        if (childOfChild != null)
                            return childOfChild;
                    }
                }
                return null;
            }
        }
        #endregion 

        private ObservableCollection<ListBoxItem> bindPageList = new ObservableCollection<ListBoxItem>();
        public bool isThumbInitialized = false;
        private const int THUMBNAIL_WIDTH = 150;
        private const int THUMBNAIL_HEIGHT = 150;
        public CPDFViewer pdfViewer;
        private List<int> pageThumbnailsToRequest = new List<int>();
        private List<int> visiblePageIndexes = new List<int>();
        private string dragingEnterPath;
        private ScrollEventType scrollType = ScrollEventType.EndScroll;
        private Point startPostion = new Point();
        private bool isFirstLoad = true;
        public bool isHided = false;
        private int zoomLevel = 2;
        private bool isZooming = false;
        private int[] thumbnailSize = { 100, 150, 200, 300, 500 };

        private bool startChoose = false;   //是否开始多选(框选)
        private int speed = 0;  //滚动速度

        //鼠标点击时在item中的位置  实现类似点哪拖哪的细节
        private double item_x;
        private double item_y;
        private int InsertIndex = -1;//插入标记代表的插入位置

        //拖动的Item
        private ListBoxItem tempItem;

        /////鼠标是否停留在item前半部
        /////显示在前半部时，获取的index为实际索引值
        /////显示在后半部时，获取的index需要+1
        //private bool isFrontHalf = false;

        private bool isDraging = false;

        //是否正在从外部拖入文件
        private bool isDragingEnter = false;

        //用来辅助实现多选后再单选的问题
        private bool iskeyDown = false;

        //页面顺序或总数改变的情况
        public event RoutedEventHandler PageMoved;

        //缩放时
        public event RoutedEventHandler ChangedZoomFactor;


        private bool isFirstScrollChange = true;
        private bool isFirstSizeChange = true;
        public string SelectedItemsRange
        {
            get { return GetPageParam(); }
        }

        public int SelectedIndex
        {
            get { return PageEditListBox.SelectedIndex; }
        }


        public event EventHandler ExitPageEdit;

        public CPDFPageEditControl()
        {
            InitializeComponent();
            SetLangText();
        }

        public void PageEdit(string PageEditString)
        {
            var list = GetListFromSelectedItems();

            if (PageEditString == "Insert")
            {
                CPDFPageInsertWindow pageInsertWindow = new CPDFPageInsertWindow();
                Window parentWindow = Window.GetWindow(this);
                pageInsertWindow.InitPageInsertWindow(SelectedIndex, pdfViewer.Document.PageCount);
                pageInsertWindow.DialogClosed += PageInsertWindow_DialogClosed;
                pageInsertWindow.Owner = parentWindow;
                pageInsertWindow.ShowDialog();
            }
            else if (PageEditString == "Replace")
            {
                DoReplace();
            }
            else if (PageEditString == "Extract")
            {
                CPDFPageExtractWindow pageExtractWindow = new CPDFPageExtractWindow();
                Window parentWindow = Window.GetWindow(this);
                pageExtractWindow.InitPageExtractWindow(SelectedItemsRange, pdfViewer.Document.PageCount);
                pageExtractWindow.DialogClosed += PageExtractWindow_DialogClosed;
                pageExtractWindow.Owner = parentWindow;
                pageExtractWindow.ShowDialog();


            }
            else if (PageEditString == "Copy")
            {
                DoCopy();
                DoPaste();
                ViewportHelper.CopyDoc = null;
            }
            else if (PageEditString == "Rotate")
            {
                DoRotate(90);
            }
            else if (PageEditString == "Delete")
            {
                DoDelete(list, true);
            }
        }

        private void PageInsertWindow_DialogClosed(object sender, InsertDialogCloseEventArgs e)
        {
            InsertEventClass result = e.DialogResult;
            if (result != null)
            {
                DoInsert(result);
            }
            else
            {

            }
        }

        private void PageExtractWindow_DialogClosed(object sender, ExtractDialogCloseEventArgs e)
        {
            ExtractEventClass result = e.DialogResult;
            if (result != null)
            {
                DoExtract(result);
            }
            else
            {

            }
        }

        private void SetLangText()
        {

        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            // 在定时器触发后，将 Border 控件隐藏
            AlertBorder.Visibility = Visibility.Collapsed;

            // 停止定时器
            DispatcherTimer timer = (DispatcherTimer)sender;
            timer.Stop();
            timer.Tick -= Timer_Tick;
        }

        #region 快捷键绑定
        private void CommandBinding_Executed_LeftRotate(object sender, ExecutedRoutedEventArgs e)
        {
            DoRotate(-90);
        }

        private void CommandBinding_Executed_RightRotate(object sender, ExecutedRoutedEventArgs e)
        {
            DoRotate(90);
        }

        private void CommandBinding_Executed_Delete(object sender, ExecutedRoutedEventArgs e)
        {
            var list = GetListFromSelectedItems();
            DoDelete(list, true);
        }

        private void CommandBinding_Executed_Copy(object sender, ExecutedRoutedEventArgs e)
        {
            DoCopy();
            DoPaste();
            ViewportHelper.CopyDoc = null;
        }

        #endregion

        #region Event
        private void PdfViewer_InfoChanged(object sender, KeyValuePair<string, object> e)
        {
            if (e.Key == "RenderNum" && this.Visibility == Visibility.Visible && !isHided)
            {
                UpdateSelectedIndex();
            }
        }

        private void _pdfviewer_OnThumbnailGenerated(int pageIndex, byte[] thumb, int w, int h)
        {
            try
            {
                if (PageEditListBox.Items.IsEmpty)
                {
                    return;
                }
                ScrollViewer sv = GetScrollHost(PageEditListBox);

                //ListBoxItem item = PageEditListBox.Items[pageIndex] as ListBoxItem;
                ListBoxItem listboxitem = PageEditListBox.ItemContainerGenerator.ContainerFromIndex(pageIndex) as ListBoxItem;
                if (ViewportHelper.IsInViewport(sv, listboxitem))
                {
                    Debug.WriteLine("Got thumbnail for page {0}. It is visible, so adding thumb", pageIndex);
                    PixelFormat fmt = PixelFormats.Bgra32;
                    BitmapSource bps = BitmapSource.Create(w, h, 96.0, 96.0, fmt, null, thumb, (w * fmt.BitsPerPixel + 7) / 8);

                    Image image = GetImageElement(PageEditListBox.Items[pageIndex] as ListBoxItem);
                    image.Source = bps;
                }
                else
                {
                    Debug.WriteLine("Got thumbnail for page {0}. It is NOT visible, so ignoring thumb", pageIndex);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
            }
        }
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            PageEditListBox.Focus();
            if (PageEditListBox.HasItems && pdfViewer.CurrentIndex < PageEditListBox.Items.Count)
            {
                PageEditListBox.ScrollIntoView(PageEditListBox.Items[pdfViewer.CurrentIndex < 0 ? 0 : pdfViewer.CurrentIndex]);
                PageEditListBox.SelectedIndex = pdfViewer.CurrentIndex;

            }
            if (pdfViewer != null)
            {
                pdfViewer.RenderCompleted += PdfViewer_InfoChanged;
                pdfViewer.AnnotEditHandler += PdfViewer_AnnotEditHandler;
            }
        }

        private void PdfViewer_AnnotEditHandler(object sender, List<ComPDFKitViewer.AnnotEvent.AnnotEditEvent> e)
        {
            if (e != null && e.Count > 0)
            {
                int pageIndex = e[0].PageIndex;
                RenderPage(pageIndex);
            }
        }

        private void UserControl_Unloaded(object sender, RoutedEventArgs e)
        {
            this.ReleaseMouseCapture();
            if (pdfViewer != null)
            {
                pdfViewer.AnnotEditHandler -= PdfViewer_AnnotEditHandler;
                pdfViewer.RenderCompleted -= PdfViewer_InfoChanged;
            }
        }

        private void UserControl_PreviewMouseWheel(object sender, MouseWheelEventArgs e)
        {
            if (Keyboard.IsKeyDown(Key.LeftCtrl))
            {
                var item = PageEditListBox.Items[0] as ListBoxItem;

                int oldZoomLevel = zoomLevel;

                if (e.Delta < 0)
                {
                    if (zoomLevel > 0)
                        zoomLevel--;
                }

                else
                {
                    if (zoomLevel < thumbnailSize.Length - 1 && thumbnailSize[zoomLevel + 1] <= PageEditListBox.ActualWidth - 2 * item.Margin.Left)
                        zoomLevel++;
                }

                if (zoomLevel != oldZoomLevel)
                {
                    isZooming = true;
                    RefreshThumbnail(true);
                    isZooming = false;
                }
                ItemsInViewHitTest();
                return;
            }
            if (startChoose)
            {
                ScrollViewer s = GetScrollHost(PageEditListBox);
                var vs = s.VerticalOffset;
                s.ScrollToVerticalOffset(vs - e.Delta);
                return;
            }
        }

        public void ZoomOut()
        {
            if (zoomLevel > 0)
            {
                zoomLevel--;
                isZooming = true;
                PopulateThumbnailList();
                if (pdfViewer.CurrentIndex < PageEditListBox.Items.Count)
                    PageEditListBox.ScrollIntoView(PageEditListBox.Items[pdfViewer.CurrentIndex]);
                PageEditListBox.SelectedIndex = pdfViewer.CurrentIndex;
                ItemsInViewHitTest();
                isZooming = false;
            }
        }

        public void ZoomIn()
        {
            if (zoomLevel < thumbnailSize.Length - 1)
            {
                zoomLevel++;
                isZooming = true;
                PopulateThumbnailList();
                if (pdfViewer.CurrentIndex < PageEditListBox.Items.Count)
                    PageEditListBox.ScrollIntoView(PageEditListBox.Items[pdfViewer.CurrentIndex]);
                PageEditListBox.SelectedIndex = pdfViewer.CurrentIndex;
                ItemsInViewHitTest();
                isZooming = false;
            }
        }


        public int CanZoomOutOrIn()
        {
            if (zoomLevel <= 0)
                return 0;
            else if (zoomLevel > 0 && zoomLevel < thumbnailSize.Length - 1)
                return 1;
            else
                return 2;

        }

        private void TryChangedZoomFactor()
        {
            try
            {
                if (ChangedZoomFactor != null)
                    ChangedZoomFactor.Invoke(null, null);
            }
            catch
            {

            }
        }

        /// <summary>
        ///
        /// </summary>
        /// <param name="isFromClick">是否是缩放时的刷新</param>
        public void RefreshThumbnail(bool isZooming = false)
        {
            try
            {
                if (pdfViewer == null) return;
                //确保缩放后选中项不发生改变
                var items = GetListFromSelectedItems();
                PopulateThumbnailList();
                if (pdfViewer.CurrentIndex >= PageEditListBox.Items.Count)
                {
                    PageEditListBox.ScrollIntoView(PageEditListBox.Items[PageEditListBox.Items.Count - 1]);
                }
                else
                    PageEditListBox.ScrollIntoView(PageEditListBox.Items[pdfViewer.CurrentIndex]);
                if (!isZooming)
                    PageEditListBox.SelectedIndex = pdfViewer.CurrentIndex;
                else
                {
                    PageEditListBox.SelectedItems.Clear();
                    for (int i = 0; i < items.Count; i++)
                    {
                        int index = int.Parse(items[i].Tag.ToString());
                        if (index < PageEditListBox.Items.Count)
                            PageEditListBox.SelectedItems.Add(PageEditListBox.Items[index]);
                    }
                }
                ItemsInViewHitTest();
                RefreshBookMarkList();
            }
            catch { }
        }

        private void PageEditListBox_ScrollChanged(object sender, ScrollChangedEventArgs e)
        {
            ItemsInViewHitTest();//第一次加载时触发的Scollchange 直接刷新界面，减少白板显示时间

            if (startChoose)
            {
                DoSelectItems();
            }

            if (isFirstScrollChange)
            {
                isFirstScrollChange = false;
                TryChangedZoomFactor();
                return;
            }
        }

        private void DoSelectItems()
        {
            var s = GetScrollHost(PageEditListBox);
            Point start = new Point();
            start = new Point(startPostion.X, startPostion.Y - s.VerticalOffset);//通过 实时的垂直偏移量和第一次的偏移量抵消，来获取准确的垂直偏移值。
            var rec = new Rect(start, Mouse.GetPosition(PageEditListBox));
            ChooseRect.Margin = new Thickness(rec.Left, rec.Top, 0, 0);
            ChooseRect.Width = rec.Width;
            ChooseRect.Height = rec.Height;
            ChooseRect.Visibility = Visibility.Visible;
            for (int i = 0; i < PageEditListBox.Items.Count; i++)//检测所有内容是否在矩形框中
            {
                var _item = PageEditListBox.Items[i] as ListBoxItem;
                //通过这一步来避免重复误选中
                var parent = Utils.FindVisualParent<VirtualizingWrapPanel>(_item);
                if (parent == null)
                    continue;

                var v = VisualTreeHelper.GetOffset(_item);
                if (rec.IntersectsWith(new Rect(v.X, v.Y, _item.ActualWidth, _item.ActualHeight)))
                {
                    PageEditListBox.SelectedItems.Add(_item);
                }
                else
                {
                    PageEditListBox.SelectedItems.Remove(_item);
                }
            }
            return;
        }

        private void PageEditListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            e.Handled = true;
        }

        private void PageEditListBox_Scroll(object sender, System.Windows.Controls.Primitives.ScrollEventArgs e)
        {
            scrollType = e.ScrollEventType;
            if (scrollType != ScrollEventType.EndScroll || isZooming)
                return;
            ItemsInViewHitTest();
        }

        private void PageEditListBox_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.LeftShift || e.Key == Key.LeftCtrl)
            {
                iskeyDown = true;
                return;
            }
        }

        private void PageEditListBox_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            iskeyDown = true;
            var pos = e.GetPosition(PageEditListBox);
            HitTestResult result = VisualTreeHelper.HitTest(PageEditListBox, pos);
            if (result == null)
            {
                return;
            }
            //未选中item  并且不是点击滑轨时 开始框选
            var listBoxItem = Utils.FindVisualParent<ListBoxItem>(result.VisualHit);
            var scroller = Utils.FindVisualParent<ScrollBar>(result.VisualHit);
            if (listBoxItem == null)
            {
                if (scroller != null)
                {
                    startChoose = false;
                    return;
                }
                //点击空白处时开始框选
                startChoose = true;
                PageEditListBox.SelectedItems.Clear();
                startPostion = e.GetPosition(PageEditListBox);
                startPostion = new Point(startPostion.X, startPostion.Y + GetWrapPanel(PageEditListBox).VerticalOffset);
                iskeyDown = false;
                Mouse.Capture(PageEditListBox);
                return;
            }
            startChoose = false;//选中了item 时，不能框选
            if (listBoxItem.IsSelected == true && !Keyboard.IsKeyDown(Key.LeftCtrl))
            {
                //  isDraging = true;
                e.Handled = true;
            }
        }

        private void PageEditListBox_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            startChoose = false;
            ChooseRect.Visibility = Visibility.Collapsed;
            var pos = e.GetPosition(PageEditListBox);
            HitTestResult result = VisualTreeHelper.HitTest(PageEditListBox, pos);
            if (result == null)
            {
                return;
            }

            var listBoxItem = Utils.FindVisualParent<ListBoxItem>(result.VisualHit);
            if (listBoxItem == null)
            {
                return;
            }

            // shift多选后，再次点击时，清空选中项，高亮当前项 按下ctrl时，不处理
            //因为listView 本身在选择项方面相当别扭，所以常见的按键模式都要逐个测试，并借助较多的限定条件才能实现
            if (iskeyDown && !Keyboard.IsKeyDown(Key.LeftCtrl) && !Keyboard.IsKeyDown(Key.LeftShift))
            {
                //IskeyDown = false;
                PageEditListBox.SelectedItems.Clear();
                PageEditListBox.SelectedItem = listBoxItem;
                listBoxItem.IsSelected = true;
                return;
            }
        }

        private void PageEditListBox_Drop(object sender, DragEventArgs e)
        {
            if (!isDraging)
            {
                MidLane.Visibility = Visibility.Collapsed;
                return;
            }
            if (isDragingEnter)
            {
                //在当前位置插入整个pdf
                CPDFDocument dragDoc = CPDFDocument.InitWithFilePath(dragingEnterPath);
                if (dragingEnterPath.Substring(dragingEnterPath.LastIndexOf(".")).ToLower() == ".pdf")
                {
                    if (dragDoc != null)
                    {
                        int index = InsertIndex == -1 ? 0 : InsertIndex;
                        pdfViewer.Document.ImportPagesAtIndex(dragDoc, "1-" + dragDoc.PageCount, index);
                        PopulateThumbnailList();
                        ItemsInViewHitTest();

                        pdfViewer.UndoManager.ClearHistory();
                        pdfViewer.UndoManager.CanSave = true;
                        pdfViewer.ReloadDocument();
                        PageEditListBox.ScrollIntoView(PageEditListBox.SelectedItem as ListBoxItem);
                        PageMoved.Invoke(this, new RoutedEventArgs());
                        dragDoc.Release();
                    }
                    else
                    {
                    }
                    //提示文档损坏无法打开
                }
                else if (!string.IsNullOrEmpty(dragingEnterPath))
                {
                    //其他文件  则新增一个页签打开
                    //    DragAddTab.Invoke(dragingEnterPath, new RoutedEventArgs());//底层库需要加一个  Load(TPDFDocument)的接口
                }
                MidLane.Visibility = Visibility.Collapsed;
                isDragingEnter = false;
                dragingEnterPath = null;
                return;
            }
            var pos = e.GetPosition(PageEditListBox);
            var result = VisualTreeHelper.HitTest(PageEditListBox, pos);
            if (result == null)
            {
                //超出当前可控区域
                MidLane.Visibility = Visibility.Collapsed;
                isDraging = false;
                return;
            }
            //查找元数据
            var sourcePerson = e.Data.GetData(typeof(StackPanel)) as StackPanel;
            if (sourcePerson == null)
            {
                MidLane.Visibility = Visibility.Collapsed;
                isDraging = false;
                return;
            }
            //查找目标数据
            int targetindex = 0;//目标插入位置
            if (InsertIndex != -1)
            {
                //往前移动时  此index  不是准确的，需要处理++
                targetindex = InsertIndex;
            }
            else//基本不会命中  仅作为保险措施
            {
                var listBoxItem = Utils.FindVisualParent<ListBoxItem>(result.VisualHit);
                if (listBoxItem == null)
                {
                    ////鼠标停留在两个item之间或其他无效区域  暂时不做处理(比较麻烦)
                    MidLane.Visibility = Visibility.Collapsed;
                    isDraging = false;
                    return;
                }
                var targetPerson = listBoxItem;
                targetPerson.Opacity = 1;
                sourcePerson.Opacity = 1;
                if (ReferenceEquals(targetPerson, sourcePerson))
                {
                    MidLane.Visibility = Visibility.Collapsed;
                    isDraging = false;
                    return;
                }

                targetindex = PageEditListBox.Items.IndexOf(targetPerson);
            }

            List<ListBoxItem> list = new List<ListBoxItem>();
            List<int> sourceindex = new List<int>();//需要保存每个页面对应的位置
            //开始排序
            List<int> pages = new List<int>();
            //要先对所有选中项 根据页码排序
            for (int i = 0; i < PageEditListBox.SelectedItems.Count; i++)
            {
                var pageindex = PageEditListBox.Items.IndexOf(PageEditListBox.SelectedItems[i] as ListBoxItem);
                pages.Add(pageindex);//存入的为页码索引值
            }
            pages.Sort();
            if (pages.Count <= 0)
            {
                MidLane.Visibility = Visibility.Collapsed;
                isDraging = false;
                return;
            }

            //要考虑每一次交换都会导致局部页码发生改变
            //每次整体往后移动时，先移动大页码；整体往前移动时，先移动小页码；往中间移动时，再按上述两种情况分别移动
            if (targetindex <= pages[0])// 目标位置在所有选中内容左边,整体前移动  优先先判断左移的情况
            {
                sourceindex.Add(-1);
                list = new List<ListBoxItem>();
                for (int i = 0; i < pages.Count; i++)
                {
                    list.Add(PageEditListBox.Items[pages[i]] as ListBoxItem);
                    sourceindex.Add(pages[i]);
                    DragToSort(pages[i], targetindex + i);
                }
            }
            else if (targetindex > pages[pages.Count - 1])//目标位置在所有选中内容右边 整体后移
            {
                sourceindex.Add(1);
                list = new List<ListBoxItem>();
                for (int i = 0; i < pages.Count; i++)
                {
                    list.Add(PageEditListBox.Items[pages[pages.Count - 1 - i]] as ListBoxItem);
                    sourceindex.Add(pages[pages.Count - 1 - i]);
                    DragToSort(pages[pages.Count - 1 - i], targetindex - 1 - i/* + (PageEditListBox.SelectedItems.Count - 1 - i)*/);
                }
            }
            else//目标位置在所有选中项中间
            {
                int i, j, k;
                for (k = 0; k < pages.Count - 1; k++)//找出PageEditListBox.Items中页码等于destpage的下标
                {
                    //这里要算入K---即前面部分的页面个数
                    if (pages[k] < targetindex && pages[k + 1] >= targetindex)
                        break;
                }

                sourceindex.Add(0);
                list = new List<ListBoxItem>();
                for (i = 0; i <= k; i++)//局部往后移动
                {
                    list.Add(PageEditListBox.Items[pages[k - i]] as ListBoxItem);
                    sourceindex.Add(pages[k - i]);
                    DragToSort(pages[k - i], targetindex - 1 - i);
                }
                for (j = i; j < pages.Count; j++)//局部往前移动
                {
                    list.Add(PageEditListBox.Items[pages[j]] as ListBoxItem);
                    sourceindex.Add(pages[j]);
                    DragToSort(pages[j], targetindex);
                    targetindex++;
                }
                sourceindex.Add(k);//往中间移时， index数组的最后一位 表示 间隔位置K
            }
            isDraging = false;
        }

        private void MidLane_Drop(object sender, DragEventArgs e)
        {
            //指针停留在中间线上后释放  也需要隐藏
            MidLane.Visibility = Visibility.Collapsed;
            this.PageEditListBox_Drop(sender, e);
        }

        private void PageEditListBox_PreviewMouseMove(object sender, MouseEventArgs e)
        {
            try
            {
                if (e.LeftButton == MouseButtonState.Pressed)
                {
                    iskeyDown = false;

                    if (startChoose)
                    {
                        var position = e.GetPosition(PageEditListBox);
                        if (position.X < 5 || position.X > PageEditListBox.ActualWidth - 5 || position.Y < 5 || position.Y > PageEditListBox.ActualHeight - 5)
                        {
                            startChoose = false;
                            ChooseRect.Visibility = Visibility.Collapsed;
                            Mouse.Capture(null);//暂时未想到靠近顶部和底部自动翻滚的好方法，只能先屏蔽这部分功能
                            return;
                        }
                        DoSelectItems();
                        return;
                    }
                    var pos = e.GetPosition(PageEditListBox);
                    if (pos.Y < 0 || pos.Y > PageEditListBox.ActualHeight)
                    {
                        MidLane.Visibility = Visibility.Collapsed;
                        return;
                    }
                    HitTestResult result = VisualTreeHelper.HitTest(PageEditListBox, pos);
                    if (result == null)
                    {
                        return;
                    }
                    var listBoxItem = Utils.FindVisualParent<ListBoxItem>(result.VisualHit);
                    if (listBoxItem == null)
                    {
                        return;
                    }
                    isDragingEnter = false;
                    tempItem = listBoxItem;
                    var panel = GetPanel(tempItem);

                    var item_pos = e.GetPosition(panel);
                    if (item_pos != null)
                    {
                        item_x = item_pos.X;
                        item_y = item_pos.Y;
                    }
                    var scroll = GetScrollHost(PageEditListBox);
                    DataObject dataObj = new DataObject(listBoxItem.Content as StackPanel);
                    DragDrop.DoDragDrop(PageEditListBox, dataObj, DragDropEffects.Move);
                    Mouse.Capture(PageEditListBox);
                    return;
                }
                ChooseRect.Visibility = Visibility.Collapsed;
                startChoose = false;
                Mouse.Capture(null);
            }
            catch (Exception ex)
            {
                MidLane.Visibility = Visibility.Collapsed;
            }
        }

        private void Grid_DragOver(object sender, DragEventArgs e)
        {
            //if (startchoose)
            //{
            //    return;
            //}
            try
            {
                //ctrl建按下 不显示插入标记和虚影  因为释放不会响应drop事件
                if (e.KeyStates == (DragDropKeyStates.ControlKey | DragDropKeyStates.LeftMouseButton) || e.KeyStates == (DragDropKeyStates.ShiftKey | DragDropKeyStates.LeftMouseButton | DragDropKeyStates.ControlKey))
                    return;

                //滚动后有 位置不准确 要减去滚动偏移量
                //控制线的位置
                var pos = e.GetPosition(PageEditListBox);
                var result = VisualTreeHelper.HitTest(PageEditListBox, pos);
                if (result == null)
                {
                    //MidLane.Visibility = Visibility.Collapsed;
                    //return;
                }

                //获取当前鼠标指针下的容器
                var listBoxItem = Utils.FindVisualParent<ListBoxItem>(result.VisualHit);
                if (listBoxItem == null)
                {
                    //MidLane.Visibility = Visibility.Collapsed;
                    //return;
                }

                #region  计算虚影位置
                //xaml层 要设置 虚影控件为左上
                double xPos, yPos;
                if (!isDragingEnter)
                {
                    Image image = GetImageElement(tempItem);//获取item 图片
                    Viewbox viewBox = (tempItem.Content as StackPanel).Children[0] as Viewbox;//获取item宽度

                    ShadowPicture.Width = viewBox.ActualWidth;
                    ShadowPicture.Height = viewBox.ActualHeight;
                    ShadowPicture.Source = image.Source;
                    xPos = e.GetPosition(PageEditListBox).X - item_x;
                    yPos = e.GetPosition(PageEditListBox).Y - item_y;
                }
                else
                {
                    var pic = ToBitmapSource(dragingEnterPath);
                    ShadowPicture.Width = pic.Width;
                    ShadowPicture.Height = pic.Height;
                    ShadowPicture.Source = pic;
                    xPos = e.GetPosition(PageEditListBox).X - pic.Width / 2;
                    yPos = e.GetPosition(PageEditListBox).Y - pic.Height / 2;
                }

                ShadowPicture.Margin = new Thickness(xPos, yPos, 0, 0);
                #endregion

                #region 计算插入标记位置
                var scroll = GetScrollHost(PageEditListBox);
                if (listBoxItem != null)
                {
                    //虚拟化影响到该值计算
                    var p = VisualTreeHelper.GetOffset(listBoxItem);//计算控件在容器中的偏移(位置)
                    MidLane.Visibility = Visibility.Visible;

                    var panel = GetWrapPanel(PageEditListBox);
                    var item = (PageEditListBox.Items[0] as ListBoxItem).DesiredSize.Width;

                    int count = (int)(panel.ViewportWidth / item);
                    var gap = (panel.ViewportWidth - count * item) / (count + 1) * 1.0;

                    MidLane.X2 = MidLane.X1 = p.X + gap / 2 + listBoxItem.DesiredSize.Width;

                    if (pos.X < p.X + gap / 2 + listBoxItem.ActualWidth / 2)
                    {
                        MidLane.X2 = MidLane.X1 = p.X - gap / 2;
                        InsertIndex = PageEditListBox.Items.IndexOf(listBoxItem);
                    }
                    else
                    {
                        InsertIndex = PageEditListBox.Items.IndexOf(listBoxItem) + 1;
                    }
                    //MidLane.Y1 = p.Y - scroll.VerticalOffset;//向下滑动后要减去滑动值
                    MidLane.Y1 = p.Y;
                    if (MidLane.Y1 < 0)//避免超出上边界
                        MidLane.Y1 = 0;
                    //MidLane.Y2 = p.Y + listBoxItem.ActualHeight - scroll.VerticalOffset;//仿智能滚动后可能会导致 垂直滚动偏量不准确
                    MidLane.Y2 = p.Y + listBoxItem.ActualHeight;
                    if (MidLane.Y2 < 0)
                    {
                        MidLane.Y2 = 0;
                    }
                }
                #endregion

                //暂时处理  鼠标移出边框时，虚影的显示问题
                if (pos.Y <= 30 || pos.Y >= PageEditListBox.ActualHeight - 10)
                {
                    MidLane.Visibility = Visibility.Collapsed;
                }

                if (pos.X <= 40 || pos.X >= scroll.ViewportWidth - 50)
                {
                    MidLane.Visibility = Visibility.Collapsed;
                }

                #region 靠近上下边界时，自动滚动，离边界越近，滚动速度越快
                speed = 0;
                if (pos.Y >= PageEditListBox.ActualHeight - 30)
                {
                    speed = 30 - (int)(PageEditListBox.ActualHeight - pos.Y);
                }
                else if (pos.Y <= 30)
                {
                    speed = (int)(pos.Y - 30);
                }

                var v = scroll.VerticalOffset;
                scroll.ScrollToVerticalOffset(v + speed);//触发连续滚动
                //SetReadModeTip(true, str, true);
                #endregion
            }
            catch (Exception ex)
            {

            }
        }

        private void ShadowPicture_Drop(object sender, DragEventArgs e)
        {
            this.PageEditListBox_Drop(sender, e);
        }

        //增加辅助判断，防止误触发 拖动排序
        private void ListBoxItem_DragLeave(object sender, DragEventArgs e)
        {
            //较大幅度拖动才能触发排序
            isDraging = true;
        }

        public void Delete_Click(object sender, RoutedEventArgs e)
        {
            if (pdfViewer == null)
                return;
            var list = GetListFromSelectedItems();
            DoDelete(list, true);
        }

        //顺时针旋转图片
        public void Rotate_Click(object sender, RoutedEventArgs e)
        {
            var item = sender as MenuItem;
            double step = 90;
            if (item.Name == "LeftRotate")
                step = -90;
            else
                step = 90;
            DoRotate(step);
            PageEditListBox.Focus();
        }


        private void ListBoxItem_PreviewMouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            e.GetPosition(PageEditListBox);
            var pos = e.GetPosition(PageEditListBox);
            HitTestResult result = VisualTreeHelper.HitTest(PageEditListBox, pos);
            if (result == null)
                return;

            var listBoxItem = Utils.FindVisualParent<ListBoxItem>(result.VisualHit);
            if (listBoxItem != null)
            {
                pdfViewer.GoToPage((int)listBoxItem.Tag);
            }
            ExitPageEdit?.Invoke(this, new EventArgs());
        }

        private void InsertBlank_Click(object sender, RoutedEventArgs e)
        {
            var send = sender as MenuItem;
        }

        private void Replace_Click(object sender, RoutedEventArgs e)
        {
            DoReplace();
        }

        public void Copy_Click(object sender, RoutedEventArgs e)
        {
            DoCopy();
            DoPaste();
            ViewportHelper.CopyDoc = null;
        }

        private void PageEditListBox_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            PageEditListBox.ScrollIntoView(PageEditListBox.SelectedItem);
        }

        private void PageEditor_IsVisibleChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            if (pdfViewer == null)
                return;
            if (isFirstLoad)
            {
                //第一次载入的时候 减少一次刷新次数
                isFirstLoad = false;
                return;
            }
            if ((bool)e.NewValue == true && pdfViewer != null && pdfViewer.CurrentIndex >= 0 /*&& (this.Parent as Grid).Name != "Thumbnail"*/)//添加一个判断条件，避免来回切换时影响缩略图显示效果
            {
                RefreshThumbnail();//保证从View 切换到PageEdit时，能及时更新 
            }
            //每次切换到页面编辑都同步选中项
            if (pdfViewer?.CurrentIndex >= PageEditListBox.Items.Count || PageEditListBox.Items.Count == 0)
                return;
            ListBoxItem item = PageEditListBox.Items[pdfViewer?.CurrentIndex == -1 ? 0 : pdfViewer.CurrentIndex] as ListBoxItem;
            PageEditListBox.ScrollIntoView(PageEditListBox.SelectedItem);
            PageEditListBox.SelectedItem = item;

        }

        private async void ListBoxItem_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            //点击刷新加载失败的图片
            if (!Keyboard.IsKeyDown(Key.LeftCtrl) && !Keyboard.IsKeyDown(Key.LeftShift))
            {
                var item = sender as ListBoxItem;
                Image img = GetImageElement(item);
                int itemindex = PageEditListBox.Items.IndexOf(item);
                if (img.Source == null)
                {
                    await pdfViewer.GetThumbnail(itemindex, (int)img.Width, (int)img.Height);
                }
                pdfViewer.GoToPage(itemindex);
            }
        }

        private void PageGrid_PreviewDragEnter(object sender, DragEventArgs e)
        {
            var file = (System.Array)e.Data.GetData(DataFormats.FileDrop);
            if (file == null || file.Length > 1)//为null 表示内部拖动 触发的
            { return; }
            foreach (var f in file)
            {
                dragingEnterPath = f.ToString();
                if (dragingEnterPath != null)
                {
                    isDragingEnter = true;
                }
            }
        }

        //右键菜单初始化
        private void ContextMenu_Loaded(object sender, RoutedEventArgs e)
        {
            if (ViewportHelper.CopyDoc == null)
            {
                var s = sender as ContextMenu;
                foreach (var item in s.Items)
                {
                    if ((item as MenuItem).Name == "Paste")
                    {
                        (item as MenuItem).IsEnabled = false;
                    }
                }
            }
            else
            {
                var s = sender as ContextMenu;
                foreach (var item in s.Items)
                {
                    if ((item as MenuItem).Name == "Paste")
                    {
                        (item as MenuItem).IsEnabled = true;
                    }
                }
            }
            if (PageEditListBox.SelectedItems.Count < 2)
            {
                var s = sender as ContextMenu;
                foreach (var item in s.Items)
                {
                    if ((item as MenuItem).Name == "Exchange")
                    {
                        (item as MenuItem).IsEnabled = false;
                    }
                }
            }
            else
            {
                var s = sender as ContextMenu;
                foreach (var item in s.Items)
                {
                    if ((item as MenuItem).Name == "Exchange")
                    {
                        (item as MenuItem).IsEnabled = true;
                    }
                }
            }
        }


        private void UserControl_KeyDown(object sender, KeyEventArgs e)
        {
            if ((Keyboard.Modifiers & ModifierKeys.Control) == ModifierKeys.Control && e.Key == Key.A)
            {
                if (PageEditListBox.SelectedItems.Count == PageEditListBox.Items.Count)
                {
                    PageEditListBox.UnselectAll();
                    e.Handled = true;
                }
            }
        }
        #endregion

        #region Method
        public void LoadThumbnails(CPDFViewer viewer)
        {
            if (viewer == null || viewer.Document == null)
                return;

            if (pdfViewer != null && viewer != pdfViewer)//创建空白文档 更新pdfviewer时
            {
                pdfViewer = viewer;
                this.PopulateThumbnailList();
                pdfViewer.OnThumbnailGenerated += _pdfviewer_OnThumbnailGenerated;
                pdfViewer.RenderCompleted += PdfViewer_InfoChanged;
                pdfViewer.AnnotEditHandler += PdfViewer_AnnotEditHandler;
            }

            pdfViewer = viewer;

            if (!isThumbInitialized)
            {
                this.PopulateThumbnailList();
                isThumbInitialized = true;
                pdfViewer.OnThumbnailGenerated += _pdfviewer_OnThumbnailGenerated;
            }
            if (pdfViewer.CurrentIndex >= 0 && PageEditListBox.Items.Count > pdfViewer.CurrentIndex)
            {
                PageEditListBox.ScrollIntoView(PageEditListBox.Items[pdfViewer.CurrentIndex]);
                (PageEditListBox.Items[pdfViewer.CurrentIndex] as ListBoxItem).IsSelected = true;
                PageEditListBox.SelectedIndex = pdfViewer.CurrentIndex;
            }
            RefreshBookMarkList();
        }

        public void PopulateThumbnailList()
        {
            visiblePageIndexes.Clear();
            bindPageList.Clear();
            // GC.Collect();

            int thumbnailWidth = thumbnailSize[zoomLevel];
            for (int i = 0; i < pdfViewer.Document.PageCount; i++)
            {
                Size pageSize = pdfViewer.Document.GetPageSize(i);
                if (pageSize.Height == 0 || pageSize.Width == 0)
                    continue;

                int imageWidth = pageSize.Width > pageSize.Height ? thumbnailWidth * 2 : (int)(pageSize.Width / pageSize.Height * thumbnailWidth * 2);
                int imageHeight = pageSize.Height > pageSize.Width ? thumbnailWidth * 2 : (int)(pageSize.Height / pageSize.Width * thumbnailWidth * 2);

                Image img = new Image()
                {
                    //Margin = new Thickness(0, 0, 5, 0),
                    Width = imageWidth,
                    Height = imageHeight,
                    Stretch = Stretch.Uniform,
                };

                Grid grid = new Grid();
                grid.Children.Add(img);
                List<Point> points = new List<Point>()
                {
                    new Point(16.75,1.25),
                    new Point(3.25,1.25 ),
                    new Point(3.25,19.4013878),
                    new Point(10,14.902),
                    new Point(16.75,19.4013878),
                };
                Polygon bookmark = new Polygon()
                {
                    Points = new PointCollection(points),
                    Fill = new SolidColorBrush((Color)ColorConverter.ConvertFromString("#FFBB00")),
                    Margin = new Thickness(0, 16, 16, 0),
                    VerticalAlignment = VerticalAlignment.Top,
                    HorizontalAlignment = HorizontalAlignment.Right,
                    Visibility = Visibility.Collapsed,
                };
                grid.Children.Add(bookmark);

                Border border = new Border()
                {
                    BorderThickness = new Thickness(2),
                    BorderBrush = new SolidColorBrush((System.Windows.Media.Color)System.Windows.Media.ColorConverter.ConvertFromString("#A0A2AE")),
                    Child = grid,
                };

                Viewbox viewBox = new Viewbox()
                {
                    Margin = new Thickness(0, 8, 0, 0),
                    Stretch = System.Windows.Media.Stretch.Uniform,
                    Width = thumbnailWidth,
                    Height = thumbnailWidth,
                    Child = border,
                };

                TextBlock text = new TextBlock()
                {
                    HorizontalAlignment = System.Windows.HorizontalAlignment.Center,
                    Text = (i + 1).ToString(),
                    Foreground = Brushes.Black,
                    Margin = new Thickness(0, 8, 0, 8),
                };

                StackPanel panel = new StackPanel();
                panel.Children.Add(viewBox);
                panel.Children.Add(text);

                ListBoxItem item = new ListBoxItem();
                item.Content = panel;
                item.Tag = i;
                item.Margin = new Thickness(6, 10, 6, 10);
                bindPageList.Add(item);
            }
            PageEditListBox.ItemsSource = bindPageList;
        }

        /// <summary>
        /// 需要先从文档层添加，再添加UI层，保证显示的页大小准确
        /// </summary>
        /// <param name="ItemIndex"></param>
        /// <returns></returns>
        public ListBoxItem GetNewItem(int ItemIndex)
        {
            int thumbnailWidth = thumbnailSize[zoomLevel];
            Size pageSize = pdfViewer.Document.GetPageSize(ItemIndex);
            if (pageSize.Width == 0 || pageSize.Height == 0)// 全选替换时，手动设定页码大小
            {
                pageSize = new Size(228, 300);
            }
            int imageWidth = pageSize.Width > pageSize.Height ? thumbnailWidth * 2 : (int)(pageSize.Width / pageSize.Height * thumbnailWidth * 2);
            int imageHeight = pageSize.Height > pageSize.Width ? thumbnailWidth * 2 : (int)(pageSize.Height / pageSize.Width * thumbnailWidth * 2);

            Image img = new Image()
            {
                // Margin = new Thickness(0, 0, 5, 0),

                Width = imageWidth,
                Height = imageHeight,
                Stretch = Stretch.Uniform,
            };

            Grid grid = new Grid();
            grid.Children.Add(img);
            List<Point> points = new List<Point>()
                {
                    new Point(16.75,1.25),
                    new Point(3.25,1.25 ),
                    new Point(3.25,19.4013878),
                    new Point(10,14.902),
                    new Point(16.75,19.4013878),
                };
            Polygon bookmark = new Polygon()
            {
                Points = new PointCollection(points),
                Fill = new SolidColorBrush((Color)ColorConverter.ConvertFromString("#FFBB00")),
                Margin = new Thickness(0, 8, 8, 0),
                VerticalAlignment = VerticalAlignment.Top,
                HorizontalAlignment = HorizontalAlignment.Right,
                Visibility = Visibility.Collapsed,
            };
            grid.Children.Add(bookmark);

            Border border = new Border()
            {
                BorderThickness = new Thickness(2),
                BorderBrush = new SolidColorBrush((System.Windows.Media.Color)System.Windows.Media.ColorConverter.ConvertFromString("#A0A2AE")),
                Child = grid,
            };

            Viewbox viewBox = new Viewbox()
            {
                Margin = new Thickness(0, 8, 0, 0),
                Stretch = System.Windows.Media.Stretch.Uniform,
                Width = thumbnailWidth,
                Height = thumbnailWidth,
                Child = border,
            };

            TextBlock text = new TextBlock()
            {
                HorizontalAlignment = System.Windows.HorizontalAlignment.Center,
                Text = (ItemIndex + 1).ToString(),
                Foreground = Brushes.Black,
                Margin = new Thickness(0, 8, 0, 8),
            };
            StackPanel panel = new StackPanel();
            panel.Children.Add(viewBox);
            panel.Children.Add(text);

            ListBoxItem item = new ListBoxItem();
            item.Content = panel;
            item.Tag = ItemIndex;
            item.Margin = new Thickness(6, 10, 6, 10);
            return item;
        }

        //UI层 在指定位置插入一定数量的空白页 在文档操作之后
        public void AddBlankPages(int pagecount, int insertindex)
        {
            for (int i = 0; i < pagecount; i++)
            {
                var item = GetNewItem(insertindex + i);
                bindPageList.Insert(insertindex + i, item);
                if (visiblePageIndexes.Contains(insertindex + i))
                    visiblePageIndexes.Remove(insertindex + i);//如果不删除，则不会显示图片
            }
            PageEditListBox.UpdateLayout();
        }

        /// <summary>
        /// 刷新指定页面的UI
        /// </summary>
        /// <param name="pageindex"></param>
        private async void RenderPage(int pageindex)
        {
            var range = GetRoughViewportRange(PageEditListBox, (PageEditListBox.Items[0] as ListBoxItem).DesiredSize, new Thickness(6, 10, 6, 10));
            if (pageindex < range.Item1 || pageindex > range.Item2)
                return;
            ListBoxItem item = PageEditListBox.Items[pageindex] as ListBoxItem;
            Image img = GetImageElement(item);
            await pdfViewer.GetThumbnail(pageindex, (int)img.Width, (int)img.Height);
        }

        private async void ItemsInViewHitTest()
        {
            if (pdfViewer == null || pdfViewer.Document == null)
                return;
            try
            {
                ScrollViewer sv = GetScrollHost(PageEditListBox);

                if (sv == null) return;

                if (VisualTreeHelper.GetParent(this) == null)
                    return;

                //List<int> pagesOnScreen = new List<int>();
                //pageThumbnailsToRequest.Clear();

                var range = GetRoughViewportRange(PageEditListBox, (PageEditListBox.Items[0] as ListBoxItem).DesiredSize, new Thickness(6, 10, 6, 10));
                for (int i = 0; i < PageEditListBox.Items.Count; ++i)
                {
                    //if (i>=pdfViewer.Document.PageCount) 
                    //{
                    //    break;
                    //}
                    ListBoxItem item = PageEditListBox.Items[i] as ListBoxItem;
                    // ListBoxItem listboxitem = PageEditListBox.ItemContainerGenerator.ContainerFromIndex(i) as ListBoxItem;
                    Image img = GetImageElement(item);
                    if (i >= (range.Item1 - 1) && i <= range.Item2 || ViewportHelper.IsInViewport(sv, item))//更改判断方式  因为BOTA缩略图不准确
                    {
                        if (img.Source == null && !visiblePageIndexes.Contains(i))
                        {
                            visiblePageIndexes.Add(i);
                            await Task.Delay(1);//有刷不出图的情况 增减页面时
                            await pdfViewer.GetThumbnail(i, (int)img.Width, (int)img.Height);
                            Debug.WriteLine("Page {0} is visible, asking for thumb", (i + 1));
                        }
                        else if (img.Source == null)
                        {
                            await pdfViewer.GetThumbnail(i, (int)img.Width, (int)img.Height);
                            Debug.WriteLine("Page {0} is visible, asking for thumb", (i + 1));
                        }
                    }
                    else
                    {
                        if (visiblePageIndexes.Contains(i))
                        {
                            Image image = GetImageElement(PageEditListBox.Items[i] as ListBoxItem);
                            if (image.Source != null)
                            {
                                image.Source = null;
                                //(image.Parent as Border).BorderBrush = Brushes.Transparent;
                                Debug.WriteLine("Page {0} is out of range, removed thumb", (i + 1));
                            }
                            else
                            {
                                Debug.WriteLine("Page {0} is out of range, but had no thumb", (i + 1));
                            }
                            visiblePageIndexes.Remove(i);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
            }
        }

        public void RefreshBookMarkList()
        {
            if (pdfViewer != null)
            {
                var booklist = pdfViewer.Document.GetBookmarkList();
                if (booklist == null) return;
                List<int> marks = new List<int>();
                for (int i = 0; i < booklist.Count; i++)
                {
                    marks.Add(booklist[i].PageIndex);
                }

                for (int k = 0; k < PageEditListBox.Items.Count; k++)
                {
                    Polygon polygon = GetBookMarkIco(PageEditListBox.Items[k] as ListBoxItem);
                    if (marks.Contains(k))
                        polygon.Visibility = Visibility.Visible;
                    else
                        polygon.Visibility = Visibility.Collapsed;
                }
            }
        }

        private void UpdateSelectedIndex()
        {
            if (PageEditListBox.Items.Count == 0)
                PageEditListBox.SelectedIndex = -1;
            else if (PageEditListBox.Items.Count <= pdfViewer.CurrentIndex)
                this.PageEditListBox.SelectedIndex = 0;
            else if (PageEditListBox.SelectedIndex != pdfViewer.CurrentIndex)
                PageEditListBox.SelectedIndex = pdfViewer.CurrentIndex;
            PageEditListBox.ScrollIntoView(PageEditListBox.Items[PageEditListBox.SelectedIndex]);
        }

        private Image GetImageElement(ListBoxItem item)
        {
            Viewbox viewBox = (item.Content as StackPanel).Children[0] as Viewbox;
            Image img = ((viewBox.Child as Border).Child as Grid).Children[0] as Image;
            return img;
        }

        private Polygon GetBookMarkIco(ListBoxItem item)
        {
            Viewbox viewBox = (item.Content as StackPanel).Children[0] as Viewbox;
            Polygon ico = ((viewBox.Child as Border).Child as Grid).Children[1] as Polygon;
            return ico;
        }

        private StackPanel GetPanel(ListBoxItem item)
        {
            StackPanel panel = item.Content as StackPanel;
            return panel;
        }

        private TextBlock GetPageNumTextBlock(ListBoxItem item)
        {
            TextBlock text = (item.Content as StackPanel).Children[1] as TextBlock;
            return text;
        }

        public VirtualizingWrapPanel GetWrapPanel(ListBox listBox)
        {
            Border border = VisualTreeHelper.GetChild(listBox, 0) as Border;
            var panel = Utils.FindVisualChild<VirtualizingWrapPanel>(border);
            return panel;
        }

        private ScrollViewer GetScrollHost(ListBox listBox)
        {
            if (VisualTreeHelper.GetChildrenCount(listBox) > 0)
            {
                int s = VisualTreeHelper.GetChildrenCount(listBox);

                Border border = VisualTreeHelper.GetChild(listBox, 0) as Border;
                if (border != null)
                {
                    return VisualTreeHelper.GetChild(border, 0) as ScrollViewer;
                }
            }
            return null;
        }

        //拖拽排序的主体方法
        private void DragToSort(int sourceindex, int targetindex)
        {
            if (targetindex == sourceindex || targetindex < 0)
            {
                MidLane.Visibility = Visibility.Collapsed;
                return;
            }
            var source = PageEditListBox.Items[sourceindex];
            bindPageList.RemoveAt(sourceindex);
            MidLane.Visibility = Visibility.Collapsed;

            bindPageList.Insert(targetindex, source as ListBoxItem);
            PageEditListBox.SelectedItems.Add(source);
            var result = pdfViewer.Document.MovePage(sourceindex, targetindex);
            if (!result)
            {
                MidLane.Visibility = Visibility.Collapsed;
                return;
            }

            pdfViewer.ReloadDocument();
            pdfViewer.GoToPage(PageEditListBox.SelectedIndex);
            pdfViewer.UndoManager.ClearHistory();
            pdfViewer.UndoManager.CanSave = true;

            ItemsInViewHitTest();
            this.PageMoved.Invoke(this, new RoutedEventArgs());
            UpdateSortedPageNum(sourceindex, targetindex);
        }

        //更新排序后的页码 适用于总页数不变的情况
        private void UpdateSortedPageNum(int sourceIndex, int targetIndex)
        {
            int sum = sourceIndex + targetIndex;
            targetIndex = targetIndex > sourceIndex ? targetIndex : sourceIndex;//保证targetindex为较大值
            sourceIndex = sum - targetIndex;
            sourceIndex = sourceIndex <= 0 ? 0 : sourceIndex;
            targetIndex = targetIndex + 1 > PageEditListBox.Items.Count ? PageEditListBox.Items.Count - 1 : targetIndex;
            for (int i = sourceIndex; i <= targetIndex; i++)
            {
                var item = PageEditListBox.Items[i] as ListBoxItem;
                item.Tag = i;
                TextBlock pagenum = GetPageNumTextBlock(item);
                pagenum.Text = (i + 1).ToString();
            }
        }

        /// <summary>
        /// 适用于页总数发生改变的页码更新
        /// </summary>
        private void UpdateAllPageNum()
        {
            for (int i = 0; i < PageEditListBox.Items.Count; i++)
            {
                var pagenum = GetPageNumTextBlock(PageEditListBox.Items[i] as ListBoxItem);
                pagenum.Text = (i + 1).ToString();
                (PageEditListBox.Items[i] as ListBoxItem).Tag = i;
            }
        }

        public void DoInsert(EventArgs e)
        {
            var data = e as InsertEventClass;
            if (data.InsertType == InsertType.BlankPages)
            {
                var size = pdfViewer.Document.GetPageSize(data.InsertIndex - 1);
                if (size.Width == 0 || size.Height == 0)
                    size = pdfViewer.Document.GetPageSize(data.InsertIndex);
                pdfViewer.Document.InsertPage(data.InsertIndex, size.Width, size.Height, null);
                RefreshThumbnail();
                PageEditListBox.SelectedItems.Clear();
                PageEditListBox.SelectedIndex = data.InsertIndex;
            }
            else if (data.InsertType == InsertType.CustomBlankPages)
            {
                pdfViewer.Document.InsertPage(data.InsertIndex, data.PageWidth, data.PageHeight, null);
                RefreshThumbnail();
                PageEditListBox.SelectedItems.Clear();
                PageEditListBox.SelectedIndex = data.InsertIndex;
            }
            else
            {
                DoAddFromOtherPdf(data.FilePath, data.PageRange, data.InsertIndex, data.Password);
            }
            UpdateAllPageNum();
            pdfViewer.UndoManager.ClearHistory();
            pdfViewer.UndoManager.CanSave = true;
            pdfViewer.ReloadDocument();
            PageEditListBox.ScrollIntoView(PageEditListBox.SelectedItem as ListBoxItem);
            ItemsInViewHitTest();//刷新图片
        }

        public void DoReplace()
        {
            if (PageEditListBox.SelectedItems.Count < 1)
            {
                ShowAlertWithTimeout(AlertType.EmptyPageAlert);
                return;
            }

            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Multiselect = false;
            dialog.Filter = "(*.pdf)|*.pdf";
            if ((bool)dialog.ShowDialog())
            {
                //用选择的整个文件替换选中页
                CPDFDocument doc = CPDFDocument.InitWithFilePath(dialog.FileName);
                //文件页数
                int pagecount = doc.PageCount;

                int insertindex = PageEditListBox.Items.IndexOf(PageEditListBox.SelectedItem);
                if (insertindex == -1)
                    insertindex = 0;

                var result = pdfViewer.Document.ImportPagesAtIndex(doc, "1-" + pagecount, insertindex);
                doc.Release();
                //UI层插入
                for (int i = 0; i < pagecount; i++)
                {
                    var item = GetNewItem(insertindex + i);
                    bindPageList.Insert(insertindex + i, item);
                    if (visiblePageIndexes.Contains(insertindex + i))
                        visiblePageIndexes.Remove(insertindex + i);//要求重新刷新图片
                }

                // RefreshThumbnail();
                pdfViewer.UndoManager.ClearHistory();
                pdfViewer.UndoManager.CanSave = true;

                var list = GetListFromSelectedItems();
                for (int i = insertindex; i < (insertindex + pagecount); i++)
                {
                    if (i < PageEditListBox.Items.Count)
                    {
                        ListBoxItem t = PageEditListBox.Items[i] as ListBoxItem;
                        PageEditListBox.SelectedItems.Add(t);
                    }
                }
                DoDelete(list, false);
                UpdateAllPageNum();//更新页码
                RefreshBookMarkList();
                ItemsInViewHitTest();//刷新图片
            }
        }

        private void DoRotate(double angleStep)
        {
            if (pdfViewer == null) return;
            if (PageEditListBox.SelectedItems.Count < 1)
            {
                ShowAlertWithTimeout(AlertType.EmptyPageAlert);
                return;
            }
            List<ListBoxItem> pages = new List<ListBoxItem>();
            for (int i = 0; i < PageEditListBox.SelectedItems.Count; i++)
            {
                var image = GetImageElement(PageEditListBox.SelectedItems[i] as ListBoxItem);

                pages.Add(PageEditListBox.SelectedItems[i] as ListBoxItem);

                double angle = (double)image.LayoutTransform.GetValue(RotateTransform.AngleProperty);
                angle += angleStep;
                angle = angle % 360;
                image.LayoutTransform = new RotateTransform(angle, image.ActualWidth / 2, image.ActualHeight / 2);
                var index = PageEditListBox.Items.IndexOf(PageEditListBox.SelectedItems[i]);
                pdfViewer.Document.RotatePage(index, (int)angleStep / 90);
                pdfViewer.Document.ReleasePages(index);
                //旋转页面后取消选中
                pdfViewer.ClearSelectPDFEdit(true);

                pdfViewer.UndoManager.ClearHistory();
                pdfViewer.UndoManager.CanSave = true;

                if (visiblePageIndexes.Contains(index))
                    visiblePageIndexes.Remove(index);
            }
            pdfViewer.ReloadDocument();
            ItemsInViewHitTest();
        }


        /// <summary>
        /// 检测文件是否重复  追加尾号
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static string CreateFilePath(string path)
        {
            int i = 1;
            string oldDestName = path;
            do
            {
                if (File.Exists(path))
                {
                    int lastDot = oldDestName.LastIndexOf('.');

                    string fileExtension = string.Empty;

                    string fileName = oldDestName;

                    if (lastDot > 0)
                    {
                        fileExtension = fileName.Substring(lastDot);

                        fileName = fileName.Substring(0, lastDot);
                    }

                    path = fileName + string.Format(@"({0})", i) + fileExtension;
                }
                ++i;

            } while (File.Exists(path));
            return path;
        }

        public void DoExtract(EventArgs e)
        {
            var data = e as ExtractEventClass;
            int pagecount = pdfViewer.Document.PageCount;
            string pageName = "";
            //获取页码范围  
            List<int> pagenums = new List<int>();
            pagenums.Clear();
            switch (data.PageMode)
            {
                case 1://全部页面
                    for (int i = 0; i < pagecount; i++)
                    {
                        pagenums.Add(i + 1);
                    }
                    pageName = "1-" + pagecount;
                    break;
                case 2://奇数页面
                    int count = (pagecount + 1) / 2;
                    for (int i = 0; i < count; i++)
                    {
                        pagenums.Add(i * 2 + 1);
                    }
                    pageName = "OddPages";
                    break;
                case 3://偶数页面
                    if (pagecount == 1)//弹窗时，已经过滤了一次，此处为以防万一。
                        return;
                    count = pagecount / 2;
                    for (int i = 0; i < count; i++)
                    {
                        pagenums.Add(i * 2 + 2);
                    }
                    pageName = "EvenPages";
                    break;
                case 4://自定义范围
                    //弹窗已经判断过文本内容是否合理
                    pagenums = data.PageParm;
                    pageName = data.PageName;
                    break;
                default:
                    break;
            }

            System.Windows.Forms.FolderBrowserDialog dialog = new System.Windows.Forms.FolderBrowserDialog();
            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                if (string.IsNullOrEmpty(dialog.SelectedPath))
                {
                    return;
                }
            }
            else
                return;
            //要显示文件路径
            string selectedfile = "";
            //根据是否单页生成文件 处理
            if (data.ExtractToSingleFile)
            {
                for (int i = 0; i < pagenums.Count; i++)
                {
                    string filename = pdfViewer.Document.FileName + " " + (pagenums[i]) + ".pdf";
                    string path = System.IO.Path.Combine(dialog.SelectedPath, filename);
                    path = CreateFilePath(path);
                    selectedfile = path;
                    CPDFDocument savedoc = CPDFDocument.CreateDocument();
                    bool result = savedoc.ImportPages(pdfViewer.Document, (pagenums[i]).ToString());
                    if (!result)
                    {
                        //SetReadModeTip(true, "提取异常，请稍后重试", false);
                        savedoc.Release();
                        continue;
                    }
                    result = savedoc.WriteToFilePath(path);
                    if (!result)
                    {
                        savedoc.Release();
                        continue;
                        //File.Delete(path);
                        // SetReadModeTip(true, "保存异常，请稍后重试", false);
                    }
                    savedoc.Release();//释放内存
                }
            }
            else//全部生成一个文件
            {
                string filename = pdfViewer.Document.FileName + " " + pageName + ".pdf";
                string path = System.IO.Path.Combine(dialog.SelectedPath, filename);
                path = CreateFilePath(path);
                selectedfile = path;
                CPDFDocument savedoc = CPDFDocument.CreateDocument();
                string range = (data.PageMode == 1 || data.PageMode == 4) ? pageName : String.Join(",", pagenums);
                bool result = savedoc.ImportPages(pdfViewer.Document, range);
                if (!result)
                {
                    return;
                    // File.Delete(path);
                }
                result = savedoc.WriteToFilePath(path);
                if (!result)
                {
                    //File.Delete(path);
                }
                //IsModefied = true;
                savedoc.Release();//释放内存
            }

            if (data.DeleteAfterExtract)//提取后删除
            {
                List<ListBoxItem> items = new List<ListBoxItem>();
                for (int i = 0; i < pagenums.Count; i++)
                {
                    ListBoxItem item = PageEditListBox.Items[pagenums[i]-1] as ListBoxItem;
                    items.Add(item);
                }

                DoDelete(items, false);
                pdfViewer.UndoManager.ClearHistory();
                pdfViewer.UndoManager.CanSave = true;
                ItemsInViewHitTest();
            }
            Process.Start(@"explorer.exe", "/select,\"" + selectedfile + "\"");

        }

        private void DoPaste()
        {
            if (pdfViewer == null) return;
            if (ViewportHelper.CopyDoc == null)
            {
                return;
            }
            int insertIndex = 0;
            //粘贴在所有选中项末尾
            for (int k = 0; k < PageEditListBox.SelectedItems.Count; k++)
            {
                if (insertIndex < PageEditListBox.Items.IndexOf(PageEditListBox.SelectedItems[k] as ListBoxItem))
                    insertIndex = PageEditListBox.Items.IndexOf(PageEditListBox.SelectedItems[k] as ListBoxItem);
            }
            insertIndex++;
            int pagecount = ViewportHelper.CopyDoc.PageCount;
            bool result = pdfViewer.Document.ImportPagesAtIndex(ViewportHelper.CopyDoc, "1-" + pagecount, insertIndex);
            if (!result)
            {
                return;
            }
            pdfViewer.ReloadDocument();
            AddBlankPages(pagecount, insertIndex);
            ItemsInViewHitTest();
            pdfViewer.UndoManager.ClearHistory();
            pdfViewer.UndoManager.CanSave = true;
            RefreshBookMarkList();

            List<ListBoxItem> pageList = new List<ListBoxItem>();
            PageEditListBox.SelectedItems.Clear();
            for (int i = 0; i < pagecount; i++)
            {
                pageList.Add(PageEditListBox.Items[insertIndex + i] as ListBoxItem);
                PageEditListBox.SelectedItems.Add(PageEditListBox.Items[insertIndex + i] as ListBoxItem);
            }

            UpdateAllPageNum();

        }

        private void DoCopy()
        {
            if (pdfViewer == null) return;
            if (PageEditListBox.SelectedItems.Count < 1)
            {
                ShowAlertWithTimeout(AlertType.EmptyPageAlert);
                return;
            }
            ViewportHelper.CopyDoc = null;
            ViewportHelper.CopyDoc = CPDFDocument.CreateDocument();
            string pageParam = GetPageParam();
            bool result = ViewportHelper.CopyDoc.ImportPages(pdfViewer.Document, pageParam);
            if (!result)
            {
                return;
            }
        }

        private void DoExchange(List<ListBoxItem> pageLists)
        {
            if (pageLists.Count <= 1)
            {
                return;
            }
            List<int> pages = new List<int>();
            for (int i = 0; i < pageLists.Count; i++)
            {
                int pageIndex = PageEditListBox.Items.IndexOf(pageLists[i] as ListBoxItem);
                pages.Add(pageIndex);
            }
            pages.Sort();

            for (int i = 0; i < pages.Count; i++)
            {
                int preindex = pages[i];
                int laterIndex = pages[pages.Count - i - 1];//尾部的索引
                if (laterIndex <= preindex)//交换完成
                    return;
                var sourceitem = PageEditListBox.Items[preindex] as ListBoxItem;
                var targetitem = PageEditListBox.Items[laterIndex] as ListBoxItem;

                bindPageList.Remove(sourceitem);
                bindPageList.Remove(targetitem);
                bindPageList.Insert(preindex, targetitem);
                bindPageList.Insert(laterIndex, sourceitem);

                pdfViewer.Document.ExchangePage(preindex, laterIndex);
                pdfViewer.UndoManager.ClearHistory();
                pdfViewer.UndoManager.CanSave = true;

                PageEditListBox.SelectedItems.Add(targetitem);
                PageEditListBox.SelectedItems.Add(sourceitem);
            }
        }

        /// <summary>
        /// 删除选中页面 bool 设置删除前是否弹窗提示
        /// </summary>
        /// <param name="tip">是否弹窗提示</param>
        private void DoDelete(List<ListBoxItem> pageLists, bool tip)
        {
            if (pageLists.Count == 0)
            {
                ShowAlertWithTimeout(AlertType.EmptyPageAlert);
                return;
            }
            if (pageLists.Count == PageEditListBox.Items.Count)
            {
                ShowAlertWithTimeout(AlertType.AllPageAlert);
                return;
            }
            Winform.DialogResult result = Winform.DialogResult.OK;
            if (tip)
            {
                result = Winform.MessageBox.Show(string.Format("Do you want to delete page {0}", GetPageParam()), "Delete", Winform.MessageBoxButtons.OKCancel, Winform.MessageBoxIcon.Warning);
            }
            if (result == Winform.DialogResult.OK || !tip)
            {
                List<int> pages = new List<int>();
                for (int i = 0; i < pageLists.Count; i++)
                {
                    var index = PageEditListBox.Items.IndexOf(pageLists[i]);
                    pages.Add(index);//存入页的index
                }
                pages.Sort();

                if (pages.Count == 0) return;
                for (int i = pages.Count - 1; i >= 0; i--)
                {
                    bindPageList.RemoveAt(pages[i]);
                }
                //文档层操
                var r = pdfViewer.Document.RemovePages(pages.ToArray());
                if (!r)
                {
                    return;
                }

                pdfViewer.UndoManager.ClearHistory();
                pdfViewer.UndoManager.CanSave = true;

                UpdateAllPageNum();
                //需要刷新 不然最后一页会出现白板
                //  ItemsInViewHitTest();
                pdfViewer.ReloadDocument();
            }
        }

        private List<ListBoxItem> GetListFromSelectedItems()
        {
            List<ListBoxItem> itemlists = new List<ListBoxItem>();
            for (int i = 0; i < PageEditListBox.SelectedItems.Count; i++)
            {
                itemlists.Add(PageEditListBox.SelectedItems[i] as ListBoxItem);
            }
            return itemlists;
        }

        /// <summary>
        /// 校验PageRange 输入是否合法，且可返回List<int> Pages  存放的索引值
        /// </summary>
        /// <param name="pageList">返回的页面集合</param>
        /// <param name="pageRange">需要判断的文本</param>
        /// <param name="count">页面总数</param>
        /// <param name="enumerationSeparator">例 new char[] { ',' }</param>
        /// <param name="rangeSeparator">例 new char[] { '-' }</param>
        /// <param name="inittag"></param>
        /// <returns></returns>
        public static bool GetPagesInRange(ref List<int> pageList, string pageRange, int count, char[] enumerationSeparator, char[] rangeSeparator, bool inittag = false)
        {
            string[] rangeSplit = pageRange.Split(enumerationSeparator);//根据分隔符 拆分字符串

            pageList.Clear();

            foreach (string range in rangeSplit)
            {
                int starttag = 1;
                if (inittag)
                {
                    starttag = 0;
                }
                if (range.Contains("-"))//连续页
                {
                    try
                    {
                        string[] limits = range.Split(rangeSeparator);//对子字符串再根据”-“ 拆分
                        if (limits.Length >= 2 && !string.IsNullOrWhiteSpace(limits[0]) && !string.IsNullOrWhiteSpace(limits[1]))
                        {
                            int start = int.Parse(limits[0]);
                            int end = int.Parse(limits[1]);

                            if ((start < starttag) || (end > count) || (start > end))
                            {
                                //throw new Exception(string.Format("Invalid page(s) in range {0} - {1}", start, end));
                                return false;
                            }

                            for (int i = start; i <= end; ++i)
                            {
                                if (pageList.Contains(i))
                                {
                                    return false;
                                }

                                pageList.Add(i - 1);
                            }
                            continue;
                        }
                    }
                    catch (Exception ex)
                    {
                        return false;
                    }
                }
                int pageNr;
                try
                {
                    // Single page
                    pageNr = int.Parse(range);//单页
                }
                catch (Exception)//格式不正确时
                {
                    return false;
                }
                if (pageNr < starttag || pageNr > count)
                {
                    return false;
                    //throw new Exception(string.Format("Invalid page {0}", pageNr));
                }
                if (pageList.Contains(pageNr))
                {
                    return false;
                    // throw new Exception(string.Format("Invalid page {0}", pageNr));
                }
                pageList.Add(pageNr - 1);
            }
            return true;
        }


        /// <summary>
        /// 从其他pdf插入页面
        /// </summary>
        /// <param name="filepath">文件路径</param>
        /// <param name="pagerange">页码范围</param>
        /// <param name="insertindex">插入位置</param>
        private void DoAddFromOtherPdf(string filepath, string pagerange, int insertindex, string password)
        {
            CPDFDocument doc = CPDFDocument.InitWithFilePath(filepath);
            if(doc.IsLocked && password != string.Empty)
            {
               if(!doc.UnlockWithPassword(password))
                {
                    return;
                }
            }
            int pagecount = 0;
            string insertRange = "";
            if (pagerange == "AllPages")
            {
                pagecount = doc.PageCount;
                insertRange = "1-" + pagecount;
            }
            else if (pagerange == "OddPages")
            {
                pagecount = (doc.PageCount + 1) / 2;
                int[] page = new int[(doc.PageCount + 1) / 2];
                for (int i = 0; i < page.Length; i++)
                {
                    page[i] = i * 2 + 1;
                }
                insertRange = string.Join(",", page);
            }
            else if (pagerange == "EvenPages")
            {
                if (doc.PageCount == 1)
                {
                    insertRange = "1";
                    pagecount = 1;
                }
                else
                {
                    pagecount = doc.PageCount / 2;
                    int[] page = new int[doc.PageCount / 2];
                    for (int i = 0; i < page.Length; i++)
                    {
                        page[i] = i * 2 + 2;
                    }
                    insertRange = string.Join(",", page);
                }
            }
            else
            {
                List<int> page = new List<int>();
                GetPagesInRange(ref page, pagerange, doc.PageCount, new char[] { ',' }, new char[] { '-' });
                insertRange = pagerange;
                pagecount = page.Count;
            }
            //先文档操作，再更新UI
            var result = pdfViewer.Document.ImportPagesAtIndex(doc, insertRange, insertindex);
            if (!result)
            {
                return;
            }
            doc.Release();
            //刷新UI
            RefreshThumbnail();
            PageEditListBox.SelectedItems.Clear();
            for (int i = insertindex; i < insertindex + pagecount; i++)
            {
                PageEditListBox.SelectedItems.Add(PageEditListBox.Items[i] as ListBoxItem);
            }
        }

        private string GetPageParam()
        {
            string pageParam = "";
            List<int> pagesList = new List<int>();

            for (int i = 0; i < PageEditListBox.SelectedItems.Count; i++)
            {
                var item = PageEditListBox.SelectedItems[i] as ListBoxItem;
                var page = GetPageNumTextBlock(item);
                if (page != null)
                {
                    pagesList.Add(int.Parse(page.Text));
                }
            }

            if (pagesList.Count != 0)
            {
                pagesList.Sort();//先对页码排序

                for (int i = 0; i < pagesList.Count; i++)
                {
                    if (i == 0)
                    {
                        pageParam += pagesList[0].ToString();
                    }
                    else
                    {
                        if (pagesList[i] == pagesList[i - 1] + 1)//页码连续
                        {
                            if (i >= 2)
                            {
                                if (pagesList[i - 1] != pagesList[i - 2] + 1)
                                    pageParam += "-";
                            }
                            else
                                pageParam += "-";

                            if (i == pagesList.Count - 1)
                            {
                                pageParam += pagesList[i].ToString();
                            }
                        }
                        else//页码不连续时
                        {
                            if (i >= 2)
                            {
                                if (pagesList[i - 1] == pagesList[i - 2] + 1)
                                    pageParam += pagesList[i - 1].ToString();
                            }
                            pageParam += "," + pagesList[i].ToString();
                        }
                    }
                }
            }
            return pageParam;
        }

        /// <summary>
        ///  返回指定路径的图标
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static BitmapSource ToBitmapSource(string path)
        {
            System.Drawing.Icon ico = System.Drawing.Icon.ExtractAssociatedIcon(path);
            System.Drawing.Bitmap bitmap = ico.ToBitmap();
            BitmapSource returnSource;
            try
            {
                returnSource = System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(bitmap.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
            }
            catch
            {
                returnSource = null;
            }
            return returnSource;
        }

        //返回当前视图中的item 范围
        private Tuple<int, int, int> GetRoughViewportRange(ListBox view, Size itemSize, Thickness itemMargin)
        {
            var scrollViewer = GetScrollHost(view);
            if (pdfViewer == null || pdfViewer.Document == null || scrollViewer == null || scrollViewer.ActualHeight == 0 || scrollViewer.ActualWidth == 0)//视图展开
                return new Tuple<int, int, int>(0, 0, 0);
            try
            {
                var currentHeight = scrollViewer.ActualHeight - view.Padding.Top;
                var currentWidth = scrollViewer.ActualWidth;
                //计算当前窗口大小能显示的行数和列数
                var columnCount = (int)(currentWidth / (itemSize.Width + itemMargin.Left));
                var rowCount = (int)Math.Ceiling(currentHeight / (itemSize.Height + itemMargin.Bottom));

                var preItemCount = (int)((scrollViewer.VerticalOffset / scrollViewer.ExtentHeight) * ((pdfViewer.Document.PageCount + columnCount - 1) / columnCount));//滑动百分比*行数 = 大概的垂直位置
                preItemCount = preItemCount * columnCount;
                var preEnd = (int)(((scrollViewer.VerticalOffset + scrollViewer.ActualHeight) / scrollViewer.ExtentHeight) * ((pdfViewer.Document.PageCount + columnCount - 1) / columnCount));
                preEnd = preEnd * columnCount + columnCount - 1;

                var middle = (int)Math.Ceiling(preItemCount + preEnd / 2d);

                return new Tuple<int, int, int>(
                    Math.Max(preItemCount, 0),
                    Math.Min(view.Items.Count, preEnd),
                    middle);
            }
            catch (Exception ex)
            {

            }
            return new Tuple<int, int, int>(0, 0, 0);
        }

        private void ShowAlertWithTimeout(AlertType alertType)
        {
            // Reset the status of the AlertBorder control.
            AlertBorder.Visibility = Visibility.Visible;
            AlertBorder.Opacity = 1.0;

            // Check the alert type and set the text of the AlertTextBlock control accordingly.
            if (alertType == AlertType.EmptyPageAlert)
            {
                AlertTextBlock.Text = "Please select at least one page.";
            }
            else if (alertType == AlertType.SinglePageALert)
            {
                AlertTextBlock.Text = "Please select above two pages.";
            }
            else if (alertType == AlertType.AllPageAlert)
            {
                AlertTextBlock.Text = "Please don't select all pages";
            }

            // Animate the opacity of the AlertBorder control to 0.0 over a period of 3 seconds.
            DoubleAnimation animation = new DoubleAnimation();
            animation.From = 1.0;
            animation.To = 0.0;
            animation.Duration = TimeSpan.FromSeconds(3);
            // Start the animation.
            AlertBorder.BeginAnimation(OpacityProperty, animation);
        }
        #endregion

        private void UserControl_MouseEnter(object sender, MouseEventArgs e)
        {
            MidLane.Visibility = Visibility.Collapsed;
        }

        private void UserControl_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            DoSelectItems();
            ItemsInViewHitTest();//第一次加载时触发的Scollchange 直接刷新界面，减少白板显示时间
        }
    }

    public class InsertEventClass : EventArgs
    {
        public InsertType InsertType = InsertType.BlankPages;
        public string FilePath = string.Empty;
        public double PageWidth = 210 * 2.54;
        public double PageHeight = 297 * 2.54;
        public string PageRange = "0";
        public int InsertIndex = 0;
        public string Password = string.Empty;
    }

    public enum InsertType
    {
        BlankPages,
        CustomBlankPages,
        FromOtherPDF
    }

    /// <summary>
    /// 用于提取的传参类
    /// </summary>
    public class ExtractEventClass : EventArgs
    {
        /// <summary>
        /// 1-全部页面 2-奇数页面 3-偶数页面 4-自定义范围
        /// </summary>
        public int PageMode;
        /// <summary>
        /// 页码 PageMode =1,2,3时 PageParm = null;
        /// </summary>
        public List<int> PageParm;
        /// <summary>
        /// 自定模式下 把文本内容传过去
        /// </summary>
        public string PageName = "";
        /// <summary>
        /// 是否拆分为单文件
        /// </summary>
        public bool ExtractToSingleFile;
        /// <summary>
        /// 提取后删除页面
        /// </summary>
        public bool DeleteAfterExtract;
    }

    public enum AlertType
    {
        EmptyPageAlert,
        SinglePageALert,
        AllPageAlert,
    }
}
